# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import gtk

from gettext import gettext as _

from .. import signals
from .. import gui

from ..helper import color
from ..com import sushi
from .inline_dialog import InlineDialog, InlineMessageDialog
from .spell_entry import SpellEntry

class TopicDialog(InlineDialog):

	"""
	[ICON] Topic for channel %s on %s          [Ok]
	       [Dis is da topic for da channel]    [Cancel]
	"""

	def __init__(self, server, channel):
		InlineDialog.__init__(self,
			buttons = (gtk.STOCK_OK, gtk.RESPONSE_OK,
						gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL),
			icon = gtk.STOCK_DIALOG_INFO)

		self.server = server
		self.channel = channel
		self._topic_changed = False

		self.table = gtk.Table(rows = 2, columns = 1)

		self.label = gtk.Label(None)
		self.label.set_markup(
				("<b>"+_("Topic for channel %(channel)s "
					"on %(server)s")+"\n</b>") %
				{ "server": server,
				  "channel": channel
				})
		self.table.attach(self.label, 0, 1, 0, 1)

		self.topicBar = SpellEntry()

		self.table.attach(self.topicBar, 0, 1, 1, 2)

		signals.connect_signal("topic", self._topic_changed_cb)
		self.topicBar.set_text(color.parse_color_codes_to_markups(
			sushi.channel_topic(server, channel)))
		self.topicBar.set_position(len(self.topicBar.get_text()))

		self.topicBar.connect("activate", self._topicBar_activate_cb)
		self.vbox.add(self.table)

	def _topicBar_activate_cb(self, edit):
		self.response(gtk.RESPONSE_OK)

	def _topic_changed_cb(self, time, server, sender, channel, topic):
		self._topic_changed = True

	def response(self, id, ignore_updated = False):

		def outdated_dialog_response_cb(dialog, id, topic_dialog):
			if id == gtk.RESPONSE_OK:
				# want apply
				topic_dialog.response(
					gtk.RESPONSE_OK,
					ignore_updated = True)

			else:
				# update the topic bar
				topic_dialog.topicBar.set_text(
					color.parse_color_codes_to_markups(
						sushi.channel_topic(
							topic_dialog.server,
							topic_dialog.channel)))

				topic_dialog.topicBar.set_position(
					len(topic_dialog.topicBar.get_text()))
				topic_dialog._topic_changed = False

			dialog.destroy()

		if id == gtk.RESPONSE_OK:
			if self._topic_changed and not ignore_updated:
				# topic changed during edit, ask user what to do

				d = InlineMessageDialog(
					_("Topic changed before."),
					_("The topic was changed before your "
					  "update. Do you want to commit the changes anyway?"),
					  buttons = (gtk.STOCK_OK, gtk.RESPONSE_OK,
					  			gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
				d.connect("response", outdated_dialog_response_cb, self)
				gui.mgmt.show_inline_dialog(d)
				return

			else:
				# apply new topic

				sushi.topic(
					self.server,
					self.channel,
					color.parse_color_markups_to_codes(
						self.topicBar.get_text()))

		InlineDialog.response(self, id)





import gtk
import gobject

from .. import gui
from ..lib import contrast
from ..helper import color as hcolor
from ..typecheck import types

class ColorDialog(object):

	""" show the available contrast colors as colored buttons
		and show a textfield with text foreground colored in
		the selected color.
	"""

	def __init__(self):
		self.builder = gui.builder.load_dialog("contrast")

		self._setup_text_tag()

		self.builder.connect_signals(self)

		self._colors = (None,None)


	def __getattr__(self, attr):
		if attr in ("builder", "color_tag", "_colors"
					"_setup_text_tag", "set_example_color",
					"get_current_color", "show_all",
					"set_current_rgb_color",
					"set_current_contrast_color",
					"contrast_color_table_color_changed",
					"colorselection_color_changed"):
			# public attributes of this object
			return super(ColorDialog,self).__getattr__(attr)
		else:
			return getattr(self.builder.get_object("contrast_dialog"), attr)


	def show_all(self):
		# FIXME:  for a reason i don't know yet, show_all() does not
		# FIXME:: apply show for the contrast_color_table, so this fixes it
		self.builder.get_object("contrast_dialog").show_all()
		self.builder.get_object("contrast_color_table").show_all()


	def _setup_text_tag(self):
		buffer = self.builder.get_object("example_text_view").get_buffer()

		self.color_tag = buffer.create_tag()

		buffer.apply_tag(self.color_tag, buffer.get_start_iter(),
			buffer.get_end_iter())


	def set_example_color(self, color):
		self.color_tag.set_property("foreground", color)


	def set_current_rgb_color(self, color):
		self.builder.get_object("colorselection").set_current_color(color)


	@types(color_code=int)
	def set_current_contrast_color(self, color_code):
		""" take a contrast color number and behave as if the specific
			color is selected.
		"""
		table = self.builder.get_object("contrast_color_table")

		button = table.get_button_by_code(color_code)
		button.clicked()


	def get_current_color(self):
		""" return a tuple with two values like this:
				(<gtk.gdk.Color()>,<contrast color code>).

			If a color is set by the colorselection, no contrast color
			is set:
				(<gtk.gdk.Color(...)>, None).

			If a contrast color is used, the first value is the RGB
			representation of the contrast color and the second value
			is the contrast color code:
				(<gtk.gdk.Color(...), <contrast color code>).

			Default is (None, None).
		"""
		return self._colors


	def contrast_color_table_color_changed(self, table, color_code):
		color = contrast.contrast_render_foreground_color(
					hcolor._get_output_bg_color(), color_code)

		# set the colorselection to the contrast color
		self.builder.get_object("colorselection").set_current_color(color)

		self._colors = (color, color_code)

		self.set_example_color(color)


	def colorselection_color_changed(self, selection):
		color = selection.get_current_color()

		self._colors = (color, None)

		self.set_example_color(color)



def setup():pass

def run(parent=None):
	d = ColorDialog()

	if parent:
		d.set_transient_for(parent)

	d.show_all()
	return d



# coding: UTF-8
"""
Copyright (c) 2008 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import gtk
import pango
import logging
import xml.sax
import xml.sax.handler

from StringIO import StringIO

from .. import gui

from ..helper.url import URLToTag
from .. import helper
from .. import config



class HTMLHandler(xml.sax.handler.ContentHandler):
	"""
	Parses HTML like strings and applies
	the tags as format rules for the given text buffer.
	"""

	def __init__(self, textbuffer, handler):
		xml.sax.handler.ContentHandler.__init__(self)

		self.textbuffer = textbuffer
		self.ignoreableEndTags = ["msg","br","su","sb"]
		self.no_cache = ["a","sb","su"]
		self.URLHandler = handler

		self._reset_values()

	def _reset_values(self):
		self.elms = []
		self.tags = []
		self.sucount = 0
		self.sbcount = 0

	def _get_cache_name(self, name, attrs):
		" return the caching name of a element for tag caching "

		def attrs_to_dict(attrs):
			return "{"+ ",".join(
				[n+":"+attrs.getValue(n) for n in attrs.getNames()]) +"}"

		if name in ("font","span"):
			return name + attrs_to_dict(attrs)

		return name

	def _get_cached(self, cachename):
		" return the cached tag for the given element or None "
		return self.textbuffer.get_tag_table().lookup(cachename)

	def _apply_tag(self, name, tag):
		""" mark the element and the tag to be added later.
			This is usually called in startElement.
		"""
		self.elms.insert(0, name)
		self.tags.insert(0, tag)

	def characters(self, text):
		""" Raw characters? Apply them (with tags, if given)
			to the text buffer
		"""
		if len(self.tags):
			# there are tags, apply them to the text
			self.textbuffer.insert_with_tags(
				self.textbuffer.get_end_iter(),
				text,
				*self.tags)
		else:
			# no tags, just add the text
			self.textbuffer.insert(
				self.textbuffer.get_end_iter(),
				text)

	def startDocument(self):
		pass

	def startElement(self, name, attrs):


		if not name in self.no_cache:

			cname = self._get_cache_name(name,attrs)

			tag = self.textbuffer.get_tag_table().lookup(cname)

			if tag: # found a already known tag, use it
				self._apply_tag(name, tag)
				return

		# handle no tag creating elements

		if name == "br":
			self.textbuffer.insert(self.textbuffer.get_end_iter(),"\n")
			return

		# create a new tag

		if name in self.no_cache:
			tag = self.textbuffer.create_tag(None)
		else:
			tag = self.textbuffer.create_tag(
				self._get_cache_name(name, attrs))

		tag.s_attribute = {} # special attribute for identifying

		tag.s_attribute[name] = True

		if name == "b":
			tag.set_property("weight", pango.WEIGHT_BOLD)

		elif name == "i":
			tag.set_property("style", pango.STYLE_ITALIC)

		elif name == "u":
			tag.set_property("underline", pango.UNDERLINE_SINGLE)

		elif name == "a":
			if self.URLHandler:
				tag.set_property("underline", pango.UNDERLINE_SINGLE)
				tag.connect("event", self.URLHandler, attrs["href"])
				tag.s_attribute["a.href"] = attrs["href"]

		elif name == "su":
			self.sucount += 1

			if self.sucount % 2 != 0:
				tag.set_property("underline", pango.UNDERLINE_SINGLE)
			else:
				tag.set_property("underline", pango.UNDERLINE_NONE)

		elif name == "sb":
			self.sbcount += 1
			if self.sbcount % 2 != 0:
				tag.set_property("weight", pango.WEIGHT_BOLD)
			else:
				tag.set_property("weight", pango.WEIGHT_NORMAL)

		elif name in ("font","span"):
			self._parseFont(tag, attrs)

		elif name == "msg":
			# start tag to avoid errors due to
			# missing overall-tag
			#
			self._parseFont(tag, attrs)


		self._apply_tag(name, tag)

	def endElement(self, name):
		if name in self.ignoreableEndTags:
			return

		try:
			i = self.elms.index(name)
		except ValueError:
			pass
		else:
			del self.elms[i]
			del self.tags[i]

	def endDocument(self):
		""" Close all special bold/underline tags. """

		if self.sbcount % 2 != 0 or self.sucount % 2 != 0:
			tag = self.textbuffer.create_tag(None)

			if self.sbcount % 2 != 0:
				tag.set_property("weight", pango.WEIGHT_NORMAL)

			if self.sucount % 2 != 0:
				tag.set_property("underline", pango.UNDERLINE_NONE)

			self.textbuffer.insert_with_tags(
				self.textbuffer.get_end_iter(),
				"",
				tag)

		self._reset_values()

	""" PARSING HELPER """

	def _parseFont(self, tag, attrs):
		if not attrs or attrs.getLength() == 0:
			return

		for name in attrs.getNames():
			if name == "weight":
				if attrs[name] == "bold":
					tag.set_property("weight", pango.WEIGHT_BOLD)
				elif attrs[name] == "normal":
					tag.set_property("weight", pango.WEIGHT_NORMAL)
			elif name == "style":
				if attrs[name] == "italic":
					tag.set_property("style", pango.STYLE_ITALIC)
				elif attrs[name] == "normal":
					tag.set_property("style", pango.STYLE_NORMAL)
			else:
				try:
					tag.set_property(name, attrs[name])
				except Exception as ex:
					logging.error("_parseFont: %s" % (ex))

class ScanHandler(xml.sax.ContentHandler):

	def __init__(self):
		xml.sax.ContentHandler.__init__(self)

class HTMLBuffer(gtk.TextBuffer):

	__gtype_name__ = "HTMLBuffer"

	global_tagtable = gtk.TextTagTable()

	def __init__(self, handler=None, tagtable=None):
		self.lines = 0

		self.group_string = None
		self.group_color = False

		if tagtable:
			self.tagtable = tagtable
		else:
			self.tagtable = self.global_tagtable

		self.URLHandler = handler

		gtk.TextBuffer.__init__(self, self.tagtable)

		self.scanner = xml.sax.make_parser()
		self.parser = xml.sax.make_parser()

		self.scanner.setContentHandler(ScanHandler())

		contentHandler = HTMLHandler(self, self.URLHandler)
		self.parser.setContentHandler(contentHandler)

	def setURLHandler(self, handler):
		self.URLHandler = handler
		self.parser.getContentHandler().URLHandler = handler

	def getURLHandler(self):
		return self.URLHandler

	def clear(self):
		""" Clears the output and resets the tag
			table to zero to save memory.
		"""
		self.set_text("")

		# clear the tagtable
		tt = self.get_tag_table()
		if tt:
			tt.foreach(lambda tag,data: data.remove(tag), tt)

	def insert(self, iter, text, *x):
		" make the buffer a ring buffer with a limit of max_output_lines "
		siter = self.get_selection_bounds()

		if siter:
			soff = siter[0].get_offset()
			ioff = siter[1].get_offset()

		gtk.TextBuffer.insert(self, iter, text, *x)

		self.lines += text.count("\n")

		try:
			max_lines = int(config.get("tekka",
				"max_output_lines"))
		except ValueError:
			max_lines = int(config.get_default("tekka",
				"max_output_lines"))

		diff = self.lines - max_lines

		if diff > 0:
			a = self.get_iter_at_line(0)
			b = self.get_iter_at_line(diff)
			self.delete(a,b)
			self.lines -= diff

		if siter:
			self.select_range(
				self.get_iter_at_offset(soff),
				self.get_iter_at_offset(ioff)
			)
		# FIXME new selection range is off by some bytes

	def insert_html(self, iter, text, group_string=None):
		""" parse text for HTML markups before adding
			it to the buffer at the given iter.

			This method is deprecated. Use insert_html.
		"""
		startoffset = iter.get_offset()

		if gtk.TextBuffer.get_char_count(self) > 0:
			text = "<br/>" + text

		text = URLToTag(text)


		if self.group_string != group_string:
			self.group_string = group_string
			self.group_color = not self.group_color

		# color the lines, group coloring by some string
		if (config.get_bool("tekka","text_rules")
		and self.group_color
		and self.group_string):
			color = helper.color.get_color_by_key("rules_color").to_string()

			text = "<msg paragraph-background='%s'>%s</msg>" % (
						color, text)
		else:
			text = "<msg>%s</msg>" % text


		def applyToParser(text):
			try:
				self.parser.parse(StringIO(text))
			except xml.sax.SAXParseException,e:
				raise Exception,\
					"%s.applyToParser: '%s' raised with '%s'." % (e, text)


		while True:
			try:
				self.scanner.parse(StringIO(text))

			except xml.sax.SAXParseException, e:
				# Exception while parsing, get (if caused by char)
				# the character and delete it. Then try again to
				# add the text.
				# If the exception is caused by a syntax error,
				# abort parsing and print the error with line
				# and position.

				pos = e.getColumnNumber()
				line = e.getLineNumber()

				if (pos-2 >= 0 and text[pos-2:pos] == "</"):
					logging.error("Syntax error on line %d, "\
						"column %d: %s\n\t%s" % (
							line,
							pos,
							text[pos:],
							text))
					return

				logging.error("HTMLBuffer: faulty char on line %d "
					"char %d ('%s')" % (line, pos, text[pos]))

				# skip the faulty char
				text = text[:pos] + text[pos+1:]

				continue

			else:

				# everything went fine, no need
				# for looping further.
				applyToParser(text)

				break



# coding: UTF-8
"""
Copyright (c) 2008 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import gtk
import os
import logging

from gettext import gettext as _

from ..gui import builder
from ..gui import mgmt
from .. import plugins as pinterface
from .. import config
from ..lib import plugin_config_dialog

widgets = None

(COL_LOADED,
 COL_AUTOLOAD,
 COL_NAME,
 COL_PATH,
 COL_VERSION,
 COL_DESC,
 COL_AUTHOR) = range(7)


def run():
	""" show the dialog """

	def dialog_response_cb(dialog, response_id):
		if response_id != 0:
			dialog.destroy()

	dialog = widgets.get_object("plugins")

	main_window = mgmt.widgets.get_object("main_window")
	dialog.set_transient_for(main_window)

	dialog.connect("response", dialog_response_cb)
	dialog.show_all()


def update_button_sensitivity(loaded):
	widgets.get_object("unloadButton").set_sensitive(loaded)
	widgets.get_object("loadButton").set_sensitive(not loaded)


def loadPlugin_clicked_cb(button):
	""" load active plugin.

		on success:
		- set loaded flag in treeview
		- update button states
	"""

	view = widgets.get_object("pluginView")
	store = view.get_model()

	path = view.get_cursor()[0]
	if not path:
		d = builder.information_dialog(
								_("No plugin selected."),
								_("You must select a plugin to load it."))
		d.connect("response", lambda w,i: w.destroy())
		d.show_all()
		return

	logging.info("loading plugin '%s'..." % (store[path][COL_NAME]))

	if pinterface.load(store[path][COL_NAME]):
		store.set(store.get_iter(path), COL_LOADED, True)
		update_button_sensitivity(loaded=True)


def unloadPlugin_clicked_cb(button):
	""" unload active plugin.

		on success:
		- set COL_LOADED in treeview False
		- update button states
	"""

	view = widgets.get_object("pluginView")
	store = view.get_model()

	path = view.get_cursor()[0]

	if not path:
		d = builder.information_dialog(
								_("No plugin selected."),
								_("You must select a plugin to unload it."))
		d.connect("response", lambda w,i: w.destroy())
		d.show_all()
		return

	logging.info("unloading plugin '%s'..." % (store[path][COL_NAME]))

	if pinterface.unload(store[path][COL_NAME]):
		store.set(store.get_iter(path), COL_LOADED, False)
		update_button_sensitivity(loaded=False)


def configureButton_clicked_cb(button):
	""" build and show configuration dialog for the currently
		selected plugin
	"""

	def dialog_response_cb(dialog, rID):
		""" apply the values and close the configuration dialog """

		dialog.save()
		dialog.destroy()

	pluginView = widgets.get_object("pluginView")
	path = pluginView.get_cursor()[0]

	plugin_name = pluginView.get_model()[path][COL_NAME]

	dialog = plugin_config_dialog.PluginConfigDialog(plugin_name)
	dialog.connect("response", dialog_response_cb)

	dialog.show_all()


def autoloadRenderer_toggled_cb(renderer, path):
	store = widgets.get_object("pluginStore")

	iter = store.get_iter(path)
	value = not store.get_value(iter, COL_AUTOLOAD)
	store.set(iter, COL_AUTOLOAD, value)

	list = config.get("autoload_plugins").items()
	name = store[path][COL_NAME]

	if not value:
		i = [i for (i,v) in list if v == name]
		if i:
			i = i[0]
		else:
			return
		config.unset("autoload_plugins", str(i))
	else:
		# activated
		config.set("autoload_plugins", str(len(list)+1), name)


def pluginView_button_press_event_cb(pluginView, event):
	""" activate the configure button if the selected plugin
		supports configuration
	"""

	if event.button == 1:
		# left click

		try:
			path = pluginView.get_path_at_pos(int(event.x),int(event.y))[0]
			pluginName = pluginView.get_model()[path][COL_NAME]

		except IndexError:
			return # no plugin selected
		except TypeError:
			return # s.a.a.

		loaded = pluginView.get_model()[path][COL_LOADED]

		update_button_sensitivity(loaded)

		options,err = pinterface.get_options(pluginName)

		if err != None:
			logging.error(err)
			mgmt.show_inline_message(
				"Error in %s" % (pluginName),
				"The config of plugin %s is faulty: %s." % (
					pluginName, err),
				dtype="error")

		logging.debug(options)

		if options:
			widgets.get_object("configureButton").set_sensitive(True)
		else:
			widgets.get_object("configureButton").set_sensitive(False)


def loadPluginList():
	view = widgets.get_object("pluginView")

	view.get_model().clear()

	paths = config.get_list("tekka", "plugin_dirs", [])

	if not paths:
		logging.error("loadPluginList: no plugin paths!")
		return False

	list = config.get("autoload_plugins", default={}).values()

	for path in paths:
		try:
			for item in os.listdir(path):

				if item[-3:] != ".py":
					continue

				# we got a module here

				loaded = pinterface.is_loaded(item)

				try:
					list.index(item)
				except ValueError:
					autoload = False
				else:
					autoload = True

				info = pinterface.get_info(item)

				if not info:
					logging.debug(
						"loadPluginList: no info for plugin '%s'" % (info))
					version = "N/A"
					desc = "N/A"
					author = "N/A"
				else:
					desc, version, author = info

				view.get_model().append(
					(loaded,
					autoload,
					item,
					os.path.join(path,item),
					version,
					desc,
					author))
		except OSError:
			continue

	return True


def setup():
	global widgets

	widgets = builder.load_dialog("plugins")

	sigdic = {
		"loadButton_clicked_cb":
			loadPlugin_clicked_cb,
		"unloadButton_clicked_cb":
			unloadPlugin_clicked_cb,
		"configureButton_clicked_cb":
			configureButton_clicked_cb,
		"pluginView_button_press_event_cb":
			pluginView_button_press_event_cb,
		"autoloadRenderer_toggled_cb":
			autoloadRenderer_toggled_cb,
	}

	widgets.connect_signals(sigdic)

	loadPluginList()



"""
Copyright (c) 2009-2010 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in the
	documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import gtk
import gobject

from ..typecheck import types

class InlineDialog(gtk.HBox):

	def __init__(self,
		icon = gtk.STOCK_DIALOG_WARNING,
		buttons = gtk.BUTTONS_CLOSE):

		"""
		 /!\ I'm a warning!  [Close]

		 /!\ I'm a long warning  [Close]
		     which got no place
			 for buttons.

	     (?) Do you want?   [Yes] [No]
		"""
		def style_set_cb (widget, style):
			if widget.setting_style:
				return

			tt = gtk.Window()
			tt.set_name("gtk-tooltip")
			tt.ensure_style()

			# set_style() may cause style-set to be triggered again.
			# It should not happen in our case, but better be safe
			# than sorry.
			widget.setting_style = True
			widget.hbox.set_style(tt.get_style().copy())
			widget.setting_style = False

			tt.destroy()

			widget.hbox.queue_draw()

		def expose_event_cb (widget, event):
			a = widget.get_allocation()

			widget.style.paint_flat_box(
				widget.window,
				gtk.STATE_NORMAL,
				gtk.SHADOW_ETCHED_IN,
				None,
				widget,
				"tooltip",
				a.x + 1,
				a.y + 1,
				a.width - 2,
				a.height - 2
			)

			return False

		def size_allocate_cb (widget, allocation):
			widget.queue_draw()
			widget.parent.queue_draw()

		gtk.HBox.__init__(self)

		self.set_property("border-width", 6)

		self.setting_style = False

		self.hbox = gtk.HBox(spacing=6)
		self.hbox.set_app_paintable(True)
		self.hbox.set_property("border-width", 6)

		# add icon
		self.icon = gtk.image_new_from_stock(icon, gtk.ICON_SIZE_DIALOG)
		self.icon.set_property("yalign", 0.0)
		self.hbox.add_with_properties(self.icon, "expand", False)

		# add vbox
		self.vbox = gtk.VBox(spacing=6)
		self.hbox.add_with_properties(self.vbox, "padding", 6)

		# add buttonbox
		self.buttonbox = gtk.VButtonBox()
		self.buttonbox.set_layout(gtk.BUTTONBOX_START)
		self.hbox.add_with_properties(self.buttonbox, "expand", False)

		if type(buttons) == gtk.ButtonsType:
			self.apply_buttons_type(buttons)
		else:
			self.add_buttons(*buttons)

		self.connect("style-set", style_set_cb)
		self.hbox.connect("expose-event", expose_event_cb)
		self.hbox.connect("size-allocate", size_allocate_cb)

		self.add(self.hbox)

	@types(btype = gtk.ButtonsType)
	def apply_buttons_type(self, btype):
		if btype == gtk.BUTTONS_NONE:
			pass

		elif btype == gtk.BUTTONS_OK:
			self.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK)

		elif btype == gtk.BUTTONS_CLOSE:
			self.add_buttons(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)

		elif btype == gtk.BUTTONS_YES_NO:
			self.add_buttons(gtk.STOCK_YES, gtk.RESPONSE_YES, gtk.STOCK_NO, gtk.RESPONSE_NO)

		elif btype == gtk.BUTTONS_OK_CANCEL:
			self.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)

	def add_buttons(self, *args):
		""" add_buttons(Label0, ResponseID0, StockID1, ResponseID1, ...) """

		if len(args) % 2 != 0:
			raise ValueError, "Not enough arguments supplied, (Button, Response,...)"

		i = 0
		while i < len(args)-1:

			try:
				stock_info = gtk.stock_lookup(args[i])
			except TypeError:
				stock_info = None

			if stock_info != None:
				# Stock item
				button = gtk.Button(stock = args[i])

			else:
				# Label
				button = gtk.Button(label = args[i])

			button.connect("clicked", lambda w,id: self.response(id), args[i+1])
			self.buttonbox.add(button)

			i += 2

	def response(self, id):
		""" button was activated, react on id """
		self.emit("response", id)

gobject.signal_new("response", InlineDialog, gobject.SIGNAL_ACTION, None, (gobject.TYPE_INT,))

class InlineMessageDialog(InlineDialog):

	def __init__(self, primary, secondary = None, *args, **kwargs):
		InlineDialog.__init__(self, *args, **kwargs)

		# add label
		self.primary_label = gtk.Label()
		self.primary_label.set_markup("<b>%s</b>" % (primary))
		self.primary_label.set_selectable(True)
		self.primary_label.set_property("xalign", 0.0)
		self.primary_label.set_property("yalign", 0.0)
		self.vbox.add(self.primary_label)

		if secondary:
			self.secondary_label = gtk.Label()
			self.secondary_label.set_markup("<small>%s</small>" % (secondary))
			self.secondary_label.set_selectable(True)
			self.secondary_label.set_property("xalign", 0.0)
			self.secondary_label.set_property("yalign", 0.0)
			self.vbox.add(self.secondary_label)
		else:
			self.secondary_label = None

		self.connect_after("style-set", self.__style_set_cb)

	def __style_set_cb(self, widget, style):
		self.primary_label.set_style(self.hbox.get_style().copy())

		if self.secondary_label:
			self.secondary_label.set_style(self.hbox.get_style().copy())








import time # strftime
import logging
import urwid

import connection
import sushi
import __main__
from extends.ListBox import ExtendedListBox

plugin_info = ("Marian Tietz",
				"1.0",
				"Adds a widget which displays every message received.")

class GeneralOutputWidget(ExtendedListBox):

	def __init__(self, plugin, *args, **kwargs):
		self.text_walker = urwid.SimpleListWalker([])

		ExtendedListBox.__init__(self, self.text_walker, *args, **kwargs)

		self.plugin = plugin
		self._connect_signals()

	def _connect_signals(self):
		self.plugin.connect_signal("message", self._message_cb)

	def _message_cb(self, timestamp, server, sender, target, message):
		if not target:
			locationFormat = server
		else:
			locationFormat = server+":"+target

		format = "[%(timestamp)s] (%(location)s) <%(nick)s> %(message)s" % {
			"timestamp": time.strftime("%H:%M", time.localtime(timestamp)),
			"location": locationFormat,
			"nick": connection.parse_from(sender)[0],
			"message": message}

		self.text_walker.append(urwid.Text(format))
		self.scroll_to_bottom()


class ProxyFrame(urwid.Frame):

	def __init__(self, *args, **kwargs):
		urwid.Frame.__init__(self, *args, **kwargs)
		logging.debug("args = %s" % (args,))

	def __getattribute__(self, attr):

		def from_frame_body(attr):
			body = urwid.Frame.__getattribute__(self, "_body")
			return getattr(body, attr)

		try:
			# return Frame attribute
			logging.debug("requesting %s" % (attr))
			return urwid.Frame.__getattribute__(self, attr)
		except AttributeError, e:
			# if this fails, return from Frame.body
			logging.debug("forwarding %s" % (attr))
			return from_frame_body(attr)



class general_output(sushi.Plugin):
	""" Nigiri plugin.

		Adds a widget bove of the body widget
		which displays every message received
		regardless of the active tab.
	"""

	def __init__(self, *args, **kwargs):
		sushi.Plugin.__init__(self, "general_output")

		self._applied = False
		self.append_general_output()

	def append_general_output(self):
		self.output = GeneralOutputWidget(self)

		body = __main__.main_window.body
		self.original_body = body

		__main__.main_window.body = ProxyFrame(body, header=urwid.LineBox(urwid.BoxAdapter(self.output, 5)))
		__main__.main_window._setup_context()
		__main__.main_window.main_loop.widget = __main__.main_window.context

		self._applied = True

	def unload(self):
		if self._applied:
			__main__.main_window.body = self.original_body
			__main__.main_window._setup_context()
		sushi.Plugin.unload(self)




# coding: UTF-8
"""
Copyright (c) 2010 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""
import time

import traceback

from gettext import gettext as _

from tekka.helper import code
from tekka.helper import color
from tekka.helper import markup

from tekka import com
from tekka import gui
from tekka import config
from tekka import signals
from tekka import commands

from tekka.com import sushi

def warnNoConnection(tab):
	if tab.is_server():
		name = tab.name
	elif tab.is_channel() or tab.is_query():
		name = tab.server.name

	gui.mgmt.show_inline_message(
		_("Warning:"),
		_("You are not connected to server %(server)s.") % {"server": name },
		dtype="warning")

def warnNotJoined(cTab):
	gui.mgmt.show_inline_message(
		_("Warning:"),
		_("The channel %(channel)s is not joined. Everything you "
		  "write will not be send.") % { "channel": cTab.name },
		dtype="warning")




def cmd_connect(currentServer, currentChannel, args):
	"""
		Connect to the given server.

		Usage: /connect <server>
	"""
	if not args:
		return gui.mgmt.myPrint("Usage: /connect <servername>")

	sushi.connect(args[0])


def cmd_quit(currentServer, currentChannel, args):
	"""
		Quit the given server with an optional reason.
		If no server is given, the current server is quit.

		Usage: /quit <server> [<reason>]
			   /quit [<reason>]
	"""
	if args:
		# /quit <server> [<reason>]
		if gui.tabs.search_tab(args[0]):
			reason = " ".join(args[1:])
			if not reason:
				reason = config.get("chatting", "quit_message", "")
			sushi.quit(args[0], reason)
		else:
			# /quit [<reason>]
			if not currentServer:
				return gui.mgmt.myPrint("Could not determine server.")
			reason = " ".join(args)
			if not reason:
				reason = config.get("chatting", "quit_message", "")
			sushi.quit(currentServer.name, reason)
	else:
		# /quit
		if not currentServer:
			return gui.mgmt.myPrint("Could not determine server.")
		sushi.quit(currentServer.name,
			config.get("chatting", "quit_message", ""))


def cmd_nick(currentServer, currentChannel, args):
	"""
		Change your current nick to the given nick.

		Usage: /nick <new nick>
	"""
	if not args:
		return gui.mgmt.myPrint("Usage: /nick <new nick>")

	if not currentServer:
		return gui.mgmt.myPrint("Can't determine my server.")

	sushi.nick(currentServer.name, args[0])


def cmd_part(currentServer, currentChannel, args):
	"""
		Part the given channel with an optional reason.
		If no channel is given, the current channel is parted.

		Usage: /part <channel> [<reason>]
			   /part [<reason>]
	"""
	if args and currentServer:
		# /part <channel> [<reason>]
		if gui.tabs.search_tab(currentServer.name, args[0]):
			reason = " ".join(args[1:])
			if not reason:
				reason = config.get("chatting", "part_message", "")
			sushi.part(currentServer.name, args[0], reason)
		else:
			# /part [<reason>]
			if not currentChannel:
				return gui.mgmt.myPrint("Could not determine channel.")
			reason = " ".join(args)
			if not reason:
				reason = config.get("chatting", "part_message", "")
			sushi.part(currentServer.name, currentChannel.name, reason)
	else:
		# /part
		if not currentChannel:
			return gui.mgmt.myPrint("Could not determine channel.")
		sushi.part(currentServer.name, currentChannel.name, config.get("chatting", "part_message", ""))


def cmd_join(currentServer, currentChannel, args):
	"""
		Joins the given channel with the optional key.
		If no channel is given, the command tries to
		join the current activated channel if it's parted.

		Usage: /join [<channel> [<key>]]
	"""
	if not currentServer:
		return gui.mgmt.myPrint("Can't determine server.")

	if not args:
		if currentChannel and not currentChannel.joined:
			return sushi.join(currentServer.name, currentChannel.name, "")
		else:
			return gui.mgmt.myPrint("Usage: /join <channel> [<key>]")

	sushi.join(currentServer.name, args[0], " ".join(args[1:]))


def cmd_action(currentServer, currentChannel, args):
	"""
		Sends an action in third person view.

		Usage: /me <text>

		Example: nemo types: /me giggles.
		Results in: nemo giggles.
	"""
	if not args:
		return gui.mgmt.myPrint("Usage: /me <text>")

	if not currentChannel:
		return gui.mgmt.myPrint("Can't find active channel.")

	sushi.action(currentServer.name, currentChannel.name, " ".join(args))


def cmd_kick(currentServer, currentTab, args):
	"""
		Kick the given user with an optional reason from the
		current channel.

		Usage: /kick <user> [<reason>]
	"""
	if not args:
		return gui.mgmt.myPrint("Usage: /kick <user> [<reason>]")

	if not currentTab or not currentTab.is_channel():
		return gui.mgmt.myPrint("You're not on a channel")

	sushi.kick(currentServer.name, currentTab.name, args[0], " ".join(args[1:]))


def cmd_mode(currentServer, currentChannel, args):
	"""
		Sets a mode on the target.

		Usage: /mode <target> (+|-)<mode> [<param>]

		Example: /mode #xesio +o nemo
		OR:	  /mode nemo +x
		OR:	  /mode #xesio +m
	"""
	if not args or len(args) < 1:
		return gui.mgmt.myPrint("Usage: /mode <target> (+|-)<mode> [<param>]")

	if not currentServer:
		return gui.mgmt.myPrint("Could not determine server.")

	if len(args) >= 2:
		# a parameter is given
		sushi.mode(currentServer.name, args[0], "%s %s" % (args[1], " ".join(args[2:])))
	else:
		sushi.mode(currentServer.name, args[0], "")


def cmd_topic(serverTab, channelTab, args):
	"""
		Sets the topic in the current channel.
		If no argument is given, print the current topic.

		Usage: /topic [<text>]
	"""
	if not channelTab or not channelTab.is_channel():
		return gui.mgmt.myPrint("No channel active.")

	if not args:
		return gui.mgmt.myPrint(
		"Topic for channel %(channel)s: '%(topic)s'" % {
			"channel": channelTab.name,
			"topic": color.parse_color_codes_to_tags(channelTab.topic)
		}, html=True)

	else:
		topic = " ".join(args)


	sushi.topic(serverTab.name, channelTab.name, topic)


def cmd_cleartopic(serverTab, channelTab, args):
	"""
		Clears the topic.

		Usage: /cleartopic
	"""
	if not channelTab or not channelTab.is_channel():
		return gui.mgmt.myPrint("No channel is active")

	sushi.topic(serverTab.name, channelTab.name, "")


def cmd_away(serverTab, channelTab, args):
	"""
		Sets you away with an optional reason.

		Usage: /away [<reason>]
	"""
	if not serverTab:
		return gui.mgmt.myPrint("Can't determine server.")

	sushi.away(serverTab.name, " ".join(args))
	serverTab.current_write(time.time(), _("You are now away."))


def cmd_back(serverTab, channelTab, args):
	"""
		Sets you back from being away.

		Usage: /back
	"""
	if not serverTab:
		return gui.mgmt.myPrint("Can't determine server.")

	sushi.back(serverTab.name)
	serverTab.current_write(time.time(), _("You are now back."))


def cmd_nickserv(serverTab, channelTab, args):
	"""
		Authenticates you at NickServ with
		the data stored in maki.

		Usage: /nickserv
	"""
	if not serverTab:
		return gui.mgmt.myPrint("Can't determine server.")

	sushi.nickserv(serverTab.name)


def cmd_ctcp(serverTab, channelTab, args):
	"""
		Sends a CTCP message to the given target.

		Usage: /ctcp <target> <message>
	"""
	if not args or len(args) < 2:
		return gui.mgmt.myPrint("Usage: /ctcp <target> <message>")

	if not serverTab:
		return gui.mgmt.myPrint("Could not determine server.")

	sushi.ctcp(serverTab.name, args[0], " ".join(args[1:]))


def cmd_names(serverTab, channelTab, args):
	"""
		Sends a NAMES request and prints the result
		as text to the channel.

		Usage: /names [<channel>]
	"""
	def request(server,channel):
		""" print the name listing in the queried tab (if it's active)
			or in the current tab
		"""
		def names_cb(time, server, channel, nicks, prefixes):

			self = names_cb

			def print_message(message):
				if self.tab.is_active():
					# print in the channel we query
					self.tab.write(time, message, "action")
				elif self.tab.is_server():
					self.tab.current_write(time, message, "action")
				else:
					# print in the current channel
					self.tab.server.current_write(time, message, "action")

			if self.first:
				print_message(_("• Begin of names"))
				self.first = False

			if not nicks: # eol
				signals.disconnect_signal("names", self)
				print_message(_("• End of names"))

			else:
				max_nicklen = max([len(n)+len(p)
					for (p,n) in self.tab.nickList.get_nicks_mode()])

				message = ""
				for i in xrange(len(nicks)):
					nick_color = color.get_nick_color(nicks[i])

					c_start = "<font foreground='%s'>" % (nick_color)
					c_end = "</font>"

					tmp = "[{prefix}{cstart}{nick}{cend}]".format(
						prefix=prefixes[i],
						nick=nicks[i],
						cstart=c_start,
						cend=c_end)

					overhead = len(tmp) - len(nicks[i]) - len(prefixes[i])

					message += "{0:{align}<{fill}}".format(tmp,
									align=" ", fill=max_nicklen + overhead)

					if (i+1) % self.max_col == 0:
						print_message(message)
						message = ""
					else:
						message += " "

				print_message(message)


		names_cb.first = True
		names_cb.max_col = int(config.get("tekka","names_columns"))
		s,c = gui.tabs.search_tabs(server, channel)
		names_cb.tab = c or s

		signals.connect_signal("names", names_cb)
		sushi.names(server, channel)

	if channelTab and not channelTab.is_channel():
		gui.mgmt.myPrint("/names can only be used on channel tabs.")
		return

	if not args:
		if channelTab:
			request(serverTab.name, channelTab.name)
		else:
			pass # which channel was meant?
	elif len(args) == 1:
		request(serverTab.name, args[0])
	else:
		gui.mgmt.myPrint("Usage: /names [<channel>]")


def cmd_notice(serverTab, channelTab, args):
	"""
		Sends a notice to the given target.
		The difference between /ctcp and /notice
		is, that /ctcp sends directly to the user
		while /notice sends the message over the
		server.

		Usage: /notice <target> <message>
	"""
	if not args or len(args) < 2:
		return gui.mgmt.myPrint("Usage: /notice <target> <message>")

	if not serverTab:
		return gui.mgmt.myPrint("Could not determine server.")

	sushi.notice(serverTab.name, args[0], " ".join(args[1:]))


def cmd_message(serverTab, channelTab, args):
	"""
		Sends a message (PRIVMSG) to the target.
		The target can be a channel or a user.

		Usage: /msg <target> <message>
	"""
	if not args or len(args) < 2:
		return gui.mgmt.myPrint("Usage: /msg <target> <message>")

	if not serverTab:
		return gui.mgmt.myPrint("Could not determine server.")

	com.sendMessage(serverTab.name, args[0], " ".join(args[1:]))


def cmd_oper(serverTab, channelTab, args):
	"""
		Authentificate as IRC operator.

		Usage: /oper <user> <pass>
	"""
	if not args or len(args) < 2:
		return gui.mgmt.myPrint("Usage: /oper <user> <pass>")

	if not serverTab:
		return gui.mgmt.myPrint("Could not determine server.")

	sushi.oper(serverTab.name, args[0], " ".join(args[1:]))


def cmd_list(serverTab, channelTab, args):
	"""
		Start a channel listing.
		If channel is given, only the channel
		is listed.

		Usage: /list [<channel>]
	"""
	import gobject

	def channelList_cb(time, server, channel, users, topic):
		""" Signal for /list command.
			Prints content of the listing.
		"""

		self = code.init_function_attrs(channelList_cb,
			_text=[],
			_line=0,
			_tab=gui.tabs.search_tab(server))

		def print_listing():
			self._tab.write_raw("<br/>".join(self._text))

			self._text = []
			self._line = 0
			return False

		if not channel and not topic and users == -1:
			# listing ended, reset variables

			def print_end():
				self._tab.write(time, "End of list.")
				return False

			if self._line > 0:
				# print rest
				gobject.idle_add(print_listing)

			gobject.idle_add(print_end)

			code.reset_function_attrs(channelList_cb)

			signals.disconnect_signal("list", channelList_cb)

		else:
			self._text.append(("• <b>%s</b><br/>\t%d "+_("User")+"<br/>"+
								"\t"+_("Topic")+": \"%s\"") % (
									markup.escape(channel),
									users,
									markup.escape(topic)))
			self._line += 1

			if self._line == 10:

				gobject.idle_add(print_listing)


	# cmd_list:

	# make the callback accessible for cmd_stop_list
	cmd_list.list_cb = channelList_cb

	if not serverTab:
		return gui.mgmt.myPrint("Could not determine server.")

	try:
		# channel specific listing?
		channel = args[0]
	except IndexError:
		# start a complete list..
		channel = ""

	serverTab.write(time.time(), "Start of list.")

	signals.connect_signal("list", channelList_cb)
	sushi.list(serverTab.name, channel)


def cmd_raw(serverTab, channelTab, args):
	"""
		Sends a command with optional args to maki
		which acts only as forwarder. The command
		goes unchanged to the server.

		Usage: /raw <command> [<further text>]
	"""
	if not args:
		return gui.mgmt.myPrint("Usage: /raw <command>")

	if not serverTab:
		return gui.mgmt.myPrint("Could not determine server.")

	# upper-case the command
	args[0] = args[0].upper()

	sushi.raw(serverTab.name, " ".join(args))


def cmd_stop_list(serverTab, channelTab, args):
	""" Aborts printing the channel list.

		Usage: /stoplist
	"""
	try:
		signals.disconnect_signal("list", cmd_list.list_cb)
	except AttributeError:
		# already stopped
		return


def cmd_whois(currentServer, currentChannel, args):
	"""
		Query a user's identity on the current server.

		Usage: /whois <user mask>
	"""
	def whois_cb(time, server, nick, message):
		""" message = "" => end of whois """

		server_tab = gui.tabs.search_tab(server)

		if message:
			server_tab.write(time,
				_(u"[%(nick)s] %(message)s") % {
					"nick": markup.escape(nick),
					"message": markup.escape(message) })
		else:
			server_tab.write(time,
				_(u"[%(nick)s] End of whois.") % {
					"nick": markup.escape(nick) })


	# begin of command function

	if not args:
		return gui.mgmt.myPrint("No server activated.")

	def handler(time, server, nick, message):
		if message:
			whois_cb(time, server, nick, message)
		else:
			signals.disconnect_signal("whois", handler)

	signals.connect_signal("whois", handler)
	sushi.whois(currentServer.name, args[0])


def cmd_query(currentServer, currentTab, args):
	"""
		Starts a query dialog with the given user.

		Usage: /query <nick>
	"""
	if not args:
		return gui.mgmt.myPrint("Usage: /query <nick>")

	if not currentServer:
		return gui.mgmt.myPrint("Can't determine server.")

	nick = args[0]

	tab = gui.tabs.search_tab(currentServer.name, nick)
	if not tab:
		# no query started

		tab = gui.tabs.create_query(currentServer, nick)
		tab.connected = True

		gui.tabs.add_tab(currentServer, tab)
		gui.shortcuts.assign_numeric_tab_shortcuts(gui.tabs.get_all_tabs())

		output = tab.window.textview.get_buffer()

		# fetch and write history to query (if any)
		tab.print_last_log()

	# jump to query
	tab.switch_to()


def cmd_clear(currentServer, currentTab, args):
	"""
		Clears the output of the current channel.

		Usage: /clear
	"""
	gui.mgmt.clear_all_outputs()


def cmd_help(currentServer, currentTab, args):
	"""
		Prints the doc-string of the given command.

		Usage: /help <command>
	"""
	if not args:
		commandList = ", ".join(commands._commands.keys())
		return gui.mgmt.myPrint("Usage: /help <command>\nAvailable commands are: %s" % (commandList))
	if commands._commands.has_key(args[0]):
		gui.mgmt.myPrint(commands._commands[args[0]].__doc__.replace("\t",""))
	else:
		gui.mgmt.myPrint("No help for %s available." % (args[0]))


def cmd_invoke_test(currentServer, currentTab, args):
	"""
		Loads a test file and executes it.

		Usage: /invoke_test <path>
	"""
	if len(args) != 1:
		return gui.myPrint("Usage: /invoke_test <path>")

	import os
	import imp

	path, file = os.path.split(args[0])

	name = file.split(".")[0]

	try:
		mod_info = imp.find_module(name, [path])
		test_module = imp.load_module(name, *mod_info)

	except ImportError as e:
		gui.mgmt.myPrint("invoke_test failed: %s" % (e))
		return

	try:
		test_module.start_test(gui)
	except Exception as e:
		gui.mgmt.myPrint("invoke_test run failed: %s\n%s" % (e,
						traceback.format_exc()))
		return


def cmd_ignore(currentServer, currentTab, args):
	"""
		Ignores a user pattern.

		Usage: /ignore <pattern>
	"""
	if not args:
		return gui.myPrint("Usage: /ignore <pattern>")
	if not currentServer:
		return gui.myPrint("Could not determine server")

	sushi.ignore(currentServer.name, args[0])
	gui.mgmt.myPrint(_("Ignoring %(user)s.") % {"user": args[0]})


def cmd_unignore(currentServer, currentTab, args):
	"""
		Unignores a user pattern.

		Usage: /unignore <pattern>
	"""
	if not args:
		return gui.myPrint("Usage: /unignore <pattern>")
	if not currentServer:
		return gui.myPrint("Could not determine server")

	sushi.unignore(currentServer.name, args[0])
	gui.mgmt.myPrint(_("Unignoring %(user)s.") % {"user":args[0]} )


def cmd_ignores(currentServer, currentTab, args):
	"""
		Shows all ignored users.

		Usage: /ignores
	"""
	if not currentServer:
		return gui.myPrint("Could not determine server")
	ignores = sushi.ignores(currentServer.name)

	if not ignores:
		gui.mgmt.myPrint(_("No users are ignored on this server."))
	else:
		gui.mgmt.myPrint(_("The following users are ignored: %(ulist)s.") % {
			"ulist": ", ".join(ignores)} )


def setup():
	_commands = {
		"connect" : cmd_connect,
		"nick" : cmd_nick,
		"part" : cmd_part,
		"join" : cmd_join,
			"j" : cmd_join,
		"me"   : cmd_action,
		"kick" : cmd_kick,
		"mode" : cmd_mode,
		"topic": cmd_topic,
		"cleartopic" : cmd_cleartopic,
		"quit" : cmd_quit,
		"away" : cmd_away,
		"back" : cmd_back,
	"nickserv" : cmd_nickserv,
		"ctcp" : cmd_ctcp,
	"invoke_test": cmd_invoke_test,
	"ignore" : cmd_ignore,
	"unignore": cmd_unignore,
	"ignores" : cmd_ignores,
		"names" :  cmd_names,
		"notice" : cmd_notice,
		"msg" : cmd_message,
		"oper" : cmd_oper,
		"list" : cmd_list,
		"raw" : cmd_raw,
		"stoplist" : cmd_stop_list,
		"whois" : cmd_whois,
		"query":  cmd_query,
		"clear":  cmd_clear,
		"help":  cmd_help
	}

	for (cmd,hdl) in _commands.items():
		commands.add_command(cmd, hdl)



# coding: UTF-8
"""
Copyright (c) 2008 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import logging

from gobject import signal_new, signal_lookup, SIGNAL_ACTION
from gtk.gdk import CONTROL_MASK, SHIFT_MASK, MOD1_MASK, keyval_from_name
from gtk import ACCEL_VISIBLE

from re import compile

shortExp = compile("(<[a-z]*>){0,1}(<[a-z]*>){0,2}(<[a-z]*>){0,3}(.*)")

regmap = {}

def removeShortcuts(accelGroup, widget):
	"""
		Removes all shortcuts registered to widget
	"""
	if not regmap.has_key(accelGroup):
		logging.error("removeShortcuts: No shortcuts registered to "
			"accel group.")
		return False

	if not regmap[accelGroup].has_key(widget):
		logging.info("removeShortcuts: No shortcuts registered to "
			"widget %s." % (widget))
		return False

	for (handler,keys,keyval,mask) in regmap[accelGroup][widget]:
		widget.remove_accelerator(accelGroup, keyval, mask)
		widget.disconnect(handler)

	del regmap[accelGroup][widget]

	return True

def removeShortcut(accelGroup, widget, shortcut):
	"""
		Removes the shortcut identified by shortcut string.
	"""
	if not regmap.has_key(accelGroup):
		logging.info("No shortcuts registered to accel group.")
		return False

	if not regmap[accelGroup].has_key(widget):
		logging.info("No shortcuts registered for widget.")
		return False

	i = 0
	for (handler, keys, keyval, mask) in regmap[accelGroup][widget]:
		if "".join(keys) == shortcut:
			widget.remove_accelerator(accelGroup, keyval, mask)
			widget.disconnect(handler)
			del regmap[accelGroup][widget][i]
			#logging.debug("deleted shortcut %s." % ("".join(keys)))
			break
		i+=1

def addShortcut(accelGroup, widget, shortcut, callback, *args):
	"""
		Adds a shortcut identified by a string
		to the widget and sets the callback
		to the shortcut.

		The shortcut string looks like this:
		<ctrl><shift>2 => (l/r)-CTRL + (l/r)-shift + 2

		There are max. 3 modifier (ctrl/shift/alt/...) allowed.
	"""

	match = shortExp.match(shortcut)

	if not match:
		logging.error("addShortcut: No pattern match for %s" % (shortcut))
		return None

	vGroups = [g for g in match.groups() if g]

	if not vGroups:
		logging.error("addShortcut: No filled groups for %s" % (shortcut))
		return None

	mask = 0

	for group in vGroups[:-1]:
		if group == "<ctrl>":
			mask |= CONTROL_MASK
		if group == "<shift>":
			mask |= SHIFT_MASK
		if group == "<alt>":
			mask |= MOD1_MASK

	key = vGroups[-1]

	if len(key) > 1:
		keyval = keyval_from_name(key)

		if not keyval:
			logging.error("addShortcut: Too much chars for (%s)." % (key))
			return None
	else:
		keyval = ord(key)

	# name like shortCut_ctrl_shift_2
	signame = "shortCut_"+"_".join([i.strip("<>") for i in vGroups])


	if not signal_lookup(signame, widget):
		signal_new(signame, widget, SIGNAL_ACTION, None, ())

	widget.add_accelerator(
		signame,
		accelGroup,
		keyval,
		mask,
		ACCEL_VISIBLE)

	handler = widget.connect(signame, callback, shortcut, *args)

	if not regmap.has_key(accelGroup):
		regmap[accelGroup] = {}

	if not regmap[accelGroup].has_key(widget):
		regmap[accelGroup][widget]=[ (handler,vGroups,keyval,mask) ]
	else:
		regmap[accelGroup][widget].append( (handler,vGroups,keyval,mask) )

	return handler





# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import gtk
import pango
from gettext import gettext as _

from . import htmlbuffer
from .. import config
from .. import gui

from ..gui.tabs import TekkaServer

def build_handler_menu(tag, widget, event, iter, attrs):

	def hide_message_cb(item, tab, msgtype):
		r_tuple = build_tuple(msgtype, tab)
		config.append_list("general_output", "filter", str(r_tuple))

	def build_tuple(msgtype, tab):
		if tab.is_channel() or tab.is_query():
			r_tuple = (str(msgtype), str(tab.server.name), str(tab.name))
		else:
			r_tuple = (str(msgtype), str(tab.name))
		return r_tuple

	tab = gui.tabs.get_tab_by_path(eval(attrs["path"]))

	if not tab:
		raise ValueError, "tab could not be retrieved (%s)" % (
			attrs["path"])

	items = []

	items.append(gtk.MenuItem(label = tab.name))
	items.append(gtk.SeparatorMenuItem())
	items.append(gtk.ImageMenuItem(gtk.STOCK_ZOOM_OUT))

	filter = config.get_list("general_output", "filter", [])
	label_s = _("Hide '%s' messages")

	items[-1].set_label(label_s % (attrs["type"]))
	items[-1].connect("activate", hide_message_cb,
		tab, attrs["type"])

	menu = gtk.Menu()
	for item in items:
		menu.add(item)

	menu.show_all()

	return menu

def go_handler(tag, widget, event, iter, attrs):

	def switch_highlight(tag, switch):
		""" switch highlighting of given tag """
		if switch:
			tag.set_property("weight", pango.WEIGHT_BOLD)
		else:
			self.tag.set_property("weight", pango.WEIGHT_NORMAL)


	self = go_handler

	# check for previous tag and unhighlight it
	if hasattr(self, "tag"):
		if self.tag != tag:
			switch_highlight(tag, False)

	# initialize (new) attributes
	self.tag = tag
	self.widget = widget
	self.event = event
	self.iter = iter
	self.path_string = attrs["path"]

	# __init__
	if not hasattr(self, "c_init"):
		self.c_init = True

		def outer_cb(*x):
			switch_highlight(self.tag, False)

		gui.widgets.get_object("main_window").connect(
			"motion-notify-event",
			outer_cb)

	# abort event handling on <a> tags
	for itag in iter.get_tags():
		try:
			itag.s_attribute["a"]
		except KeyError:
			pass
		else:
			return False

	# event handling
	if event.type == gtk.gdk.MOTION_NOTIFY:
		if event.state & gtk.gdk.BUTTON1_MASK:
			return False

		switch_highlight(tag, True)
		return True

	if event.type == gtk.gdk.BUTTON_PRESS:
		if event.button == 3:
			# right mbtn
			menu = build_handler_menu(tag, widget, event, iter, attrs)
			menu.popup(None, None, None, event.button, event.time)
			return True

	if event.type == gtk.gdk.BUTTON_RELEASE:

		# left mbtn
		if (event.button == 1
		and not widget.get_buffer().get_has_selection()):
			path = eval(self.path_string)
			gui.tabs.switch_to_path(path)


class GOHTMLHandler(htmlbuffer.HTMLHandler):

	def __init__(self, textbuffer, GOHandler, URLhandler):
		htmlbuffer.HTMLHandler.__init__(self, textbuffer, URLhandler)
		self.go_handler = GOHandler

	def startElement(self, name, attrs):
		if name == "goref":
			if self.go_handler:
				# TODO caching of goref
				tag = self.textbuffer.create_tag(None)

				tag.s_attribute = {"goref":True}

				# workaround the priority in the taglist of HTMLHandler
				# Given <msg><goref><font></font></goref></msg>
				# Results in [font, goref, msg], so goref has a higher
				# priority than font. As a result, every change made in
				# goref overrides properties of font. We don't want this,
				# so we set the goref tag to a low priority.
				tag.set_priority(0)

				tag.connect("event", self.go_handler, attrs)

				self._apply_tag(name, tag)

		else:
			htmlbuffer.HTMLHandler.startElement(self, name, attrs)


class GOHTMLBuffer(htmlbuffer.HTMLBuffer):

	__gtype_name__ = "GOHTMLBuffer"

	def __init__(self, go_handler=go_handler, handler=None):
		htmlbuffer.HTMLBuffer.__init__(self, handler)

		contentHandler = GOHTMLHandler(self, go_handler, self.URLHandler)
		self.parser.setContentHandler(contentHandler)

		self.go_handler = go_handler

	def go_insert(self, iter, text, tab, msgtype):
		""" Filter messages and insert other into the buffer """

		# check the type generally
		allowed_types = config.get_list("general_output",
			"valid_types",[])

		if msgtype not in allowed_types:
			return

		# check for special filters
		filter = config.get_list("general_output", "filter", [])

		if type(tab) == TekkaServer:
			server = tab.name
			channel = ""
		else:
			server = tab.server.name
			channel = tab.name

		for tuple_str in filter:
			try:
				r_tuple = eval(tuple_str)

			except BaseException as e:
				logging.error("Error in filter tuple '%s': %s" % (
								tuple_str, e))
				continue

			# if the rule matches, abort execution
			if r_tuple[0] == msgtype and r_tuple[-1] in (server, channel):
				return

		# not filtered, insert
		self.insert_html(iter,
			"<goref type='%s' path='%s'>%s</goref>" % (
				msgtype, tab.path, text))



import gtk
import gobject

from gettext import gettext as _

from . import contrast
from . import custom_color_button

from ..helper import color

class ContrastColorTable(gtk.Table):

	""" Display all available contrast colors as color buttons
		in n >= 1 rows depending on the column count.
		The default column count is 6.
	"""

	__gtype_name__ = "ContrastColorTable"


	def __init__(self, columns=6):
		super(ContrastColorTable, self).__init__()

		self._buttons = []

		self.set_columns(columns)

		self.contrast_color = contrast.CONTRAST_COLOR_BLACK


	def get_color_palette(self):
		return [
			contrast.CONTRAST_COLOR_BLACK,
			contrast.CONTRAST_COLOR_WHITE,
			contrast.CONTRAST_COLOR_LIGHT_GREY,
			contrast.CONTRAST_COLOR_GREY,
			contrast.CONTRAST_COLOR_DARK_GREY,
			contrast.CONTRAST_COLOR_LIGHT_GREEN,
			contrast.CONTRAST_COLOR_GREEN,
			contrast.CONTRAST_COLOR_DARK_GREEN,
			contrast.CONTRAST_COLOR_AQUA,
			contrast.CONTRAST_COLOR_CYAN,
			contrast.CONTRAST_COLOR_LIGHT_BLUE,
			contrast.CONTRAST_COLOR_BLUE,
			contrast.CONTRAST_COLOR_DARK_BLUE,
			contrast.CONTRAST_COLOR_PURPLE,
			contrast.CONTRAST_COLOR_VIOLET,
			contrast.CONTRAST_COLOR_MAGENTA,
			contrast.CONTRAST_COLOR_BROWN,
			contrast.CONTRAST_COLOR_LIGHT_BROWN,
			contrast.CONTRAST_COLOR_LIGHT_RED,
			contrast.CONTRAST_COLOR_RED,
			contrast.CONTRAST_COLOR_DARK_RED,
			contrast.CONTRAST_COLOR_ORANGE,
			contrast.CONTRAST_COLOR_YELLOW,
		]


	def get_buttons(self):
		return self._buttons


	def get_button_by_code(self, contrast_color):
		""" return the button responsible for the given color code """
		palette = self.get_color_palette()
		for i in range(len(palette)):
			if contrast_color == palette[i]:
				return self.get_buttons()[i]
		return None


	def fill(self, columns):
		""" fill the table """
		self.foreach(lambda w: self.remove(w))
		self._buttons = []

		x,y = (0,0)

		try:
			bg = color._get_output_bg_color()
		except:
			# in case of glade or test
			bg = gtk.gdk.Color("#fff")

		for code in self.get_color_palette():
			ccolor = contrast.contrast_render_foreground_color(bg, code)

			button = custom_color_button.CustomColorButton(ccolor)
			button.connect("clicked", self.button_clicked, code)

			xoptions = yoptions = gtk.FILL

			self.attach(button, x, x+1, y, y+1, xoptions, yoptions)

			self._buttons.append(button)

			x += 1

			if x == columns:
				x = 0
				y += 1


	def button_clicked(self, button, color_code):
		self.set_contrast_color(color_code)


	def change_color(self, color_code):
		self.emit("color-changed", color_code)


	def set_columns(self, columns):
		self.fill(columns)
		self._columns = columns


	def set_contrast_color(self, color_code):
		self._contrast_color = color_code
		self.change_color(color_code)


	columns = property(
		lambda s: s._columns,
		set_columns,
		doc="Number of columns per row. Default is 6.")

	gcolumns = gobject.property(
		getter=lambda s: s.columns,
		setter=set_columns,
		default=6,
		type=int)

	contrast_color = property(
		lambda s: s._contrast_color,
		set_contrast_color,
		doc="The selected color. Default is black.")


gobject.signal_new(
	"color-changed",
	ContrastColorTable,
	gobject.SIGNAL_ACTION,
	gobject.TYPE_NONE,
	(gobject.TYPE_INT,))




""" Misc. code helper """

def _generate_unique_attribute(fun):
	return "__init_%s_%s" % (fun.func_name, str(id(fun)))

def init_function_attrs(fun, **vars):
	""" Add the variables with values as attributes to the function fun
		if they do not exist and return the function.

		Usage: self = init_function_attr(myFun, a = 2, b = 3)
		Results in: self = myFun, self.a = 2, self.b = 3
	"""
	unique_attr = _generate_unique_attribute(fun)

	try:
		getattr(fun, unique_attr)
	except AttributeError:
		for (key, val) in vars.items():
			setattr(fun, key, val)
		setattr(fun, unique_attr, True)

	return fun

def reset_function_attrs(fun):
	try:
		getattr(fun, _generate_unique_attribute(fun))
	except AttributeError:
		pass
	else:
		delattr(fun, _generate_unique_attribute(fun))



"""
Copyright (c) 2010 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

""" Purpose:
	Raw config dialog for a plugin which can save the settings
	for a given plugin and displays the options the plugin offers.

	This is used by the plugin management dialog.
"""


import gtk
import gobject
from gettext import gettext as _
import logging
import json

from .. import config
from .. import plugins
from . import psushi
from .error import TekkaError

PLUGIN_OPTIONS_LENGTH = 4

class PluginConfigDialog(gtk.Dialog):

	def __init__(self, plugin_name):
		super(PluginConfigDialog, self).__init__(
			title=_("Configure %(name)s" % {"name": plugin_name}),
			buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
		)

		self.plugin_name = plugin_name
		self.plugin_options, err = plugins.get_options(plugin_name)

		if err != None:
			logging.error(err)
			gui.mgmt.show_inline_message(
				"Config dialog %s" % (plugin_name),
				"Can't load the config dialog for plugin '%s': %s" % (
					plugin_name, err),
				dtype="error")
			return


		self._data_map = {}

		self._build_interface()

		self._fill()


	def _build_interface(self):

		self.table = table = gtk.Table(
				rows=len(self.plugin_options),
				columns=2)
		table.set_property("column-spacing", 12)
		table.set_property("row-spacing", 6)

		# put another vbox arround it because
		# d.vbox.set_property("border-width",..) does not work...
		self.content_vbox = vbox = gtk.VBox()
		vbox.set_property("border-width", 12)
		vbox.pack_start(table)
		self.vbox.pack_start(vbox)

	def composit_list_widget(self, opt, label, value, dataMap):
		""" Return a widget which holds and manages a list widget """
			
		def row_content_to_entry(tv, path, column, entry):
			model = tv.get_model()
			entry.set_text(model[path][0])
			entry.grab_focus()
			model.remove(model.get_iter(path))
	
		def add_data_to_list(btn, listview, entry):
			model = listview.get_model()
			model.append(row=(entry.get_text(),))
			entry.set_text("")
	
		def delete_data_from_list(btn, listview):
			(model, iter) = listview.get_selection().get_selected()
			if iter == None:
				return
			model.remove(iter)
	
		def update_data_map(model, path, *_):
			dataMap[opt] = json.dumps([n[0] for n in model])
	
		addButton = gtk.Button(stock=gtk.STOCK_ADD)
		deleteButton = gtk.Button(stock=gtk.STOCK_REMOVE)
	
		listStore = gtk.ListStore(str)
		listRenderer = gtk.CellRendererText()
		listView = gtk.TreeView(listStore)
		listWindow = gtk.ScrolledWindow()
	
		textEntry = gtk.Entry()
	
		# list setup
		listView.insert_column_with_attributes(0, "", listRenderer, text=0)
		listView.set_property("headers-visible", False)
		
		default_list = []
		if value:
			try:
				default_list = json.loads(value)
			except Exception as e:
				default_list = []
		
		if default_list and not isinstance(default_list, list):
			logging.error("Invalid default value (not a list)")
		else:
			for item in default_list:
				listStore.append(row=(item,))
	
		# window setup
		listWindow.set_properties(hscrollbar_policy=gtk.POLICY_AUTOMATIC,
			vscrollbar_policy=gtk.POLICY_AUTOMATIC, border_width=3)
		listWindow.set_shadow_type(gtk.SHADOW_ETCHED_IN)
		listWindow.add(listView)
	
		container = gtk.VBox()
		buttonBox = gtk.HBox()
	
		buttonBox.pack_start(addButton)
		buttonBox.pack_start(deleteButton)
	
		container.pack_start(listWindow, expand=True)
		container.pack_start(textEntry, expand=False)
		container.pack_start(buttonBox, expand=False)
	
		frame = gtk.Frame(label=label)
		frame.set_shadow_type(gtk.SHADOW_ETCHED_IN)
		frame.set_property("border-width", 6)
		frame.add(container)
	
		# signals
		listView.connect("row-activated", row_content_to_entry, textEntry)
	
		listStore.connect("row-changed", update_data_map)
		listStore.connect("row-deleted", update_data_map)
	
		addButton.connect("clicked", add_data_to_list, listView, textEntry)
		deleteButton.connect("clicked", delete_data_from_list, listView)
	
		return frame,None

	def composit_map_widget(*x):
		return None

	def _fill(self):
		""" fill the dialog's content box/table with widgets
			according to the plugin options.
		"""

		cSection = plugins.get_plugin_config_section(
				self.plugin_name)

		dataMap = {} # config_key : value
		rowCount = 0

		for option in self.plugin_options:
			
			if len(option) != PLUGIN_OPTIONS_LENGTH:
				logging.error("Faulty plugin %s: Invalid plugin options count: %d" % (
					self.plugin_name, len(option)))
				return
			
			(opt, label, vtype, value) = option

			def text_changed_cb(widget, option):
				value = widget.get_text()
				dataMap[option] = value


			wLabel = gtk.Label(label+": ")
			wLabel.set_property("xalign", 0)

			widget = None

			cValue = config.get(cSection, opt) or value
			dataMap[opt] = cValue


			if vtype == psushi.TYPE_STRING:
				# Simple text entry
				widget = gtk.Entry()
				widget.set_text(cValue)

				widget.connect("changed", text_changed_cb, opt)

			elif vtype == psushi.TYPE_PASSWORD:
				# Hidden char. entry
				widget = gtk.Entry()
				widget.set_text(cValue)
				widget.set_property("visibility", False)

				widget.connect("changed", text_changed_cb, opt)

			elif vtype == psushi.TYPE_NUMBER:
				# Number entry field

				def changed_cb(widget, option):
					dataMap[option] = widget.get_value_as_int()

				widget = gtk.SpinButton()
				widget.set_range(-99999,99999)
				widget.set_increments(1, 5)
				widget.set_value(int(cValue))

				widget.connect("value-changed", changed_cb, opt)

			elif vtype == psushi.TYPE_BOOL:
				# Check button for boolean values

				def changed_cb(widget, option):
					dataMap[option] = widget.get_active()

				widget = gtk.CheckButton()
				if type(cValue) == bool:
					widget.set_active(cValue)
				else:
					widget.set_active(cValue.lower() != "false")

				widget.connect("toggled", changed_cb, opt)

			elif vtype == psushi.TYPE_CHOICE:
				# Multiple values. Stored as [0] = key and [1] = value

				def changed_cb(widget, option):
					if widget.get_active() >= 0:
						value = widget.get_model()[widget.get_active()][1]
						dataMap[option] = value
					else:
						dataMap[option] = ""

				wModel = gtk.ListStore(
						gobject.TYPE_STRING,
						gobject.TYPE_STRING)
				widget = gtk.ComboBox(wModel)

				widget.connect("changed", changed_cb, opt)

				wRenderer = gtk.CellRendererText()
				widget.pack_start(wRenderer, True)
				widget.add_attribute(wRenderer, "text", 0)

				for (key, val) in value:
					wModel.append(row = (key, val))

				# this is tricky, if there's a saved value,
				# find the matching value (second field!)
				# and set the index to that position.
				if cValue and cValue != value:
					i = 0
					for row in wModel:
						if row[1] == cValue:
							break
						i+=1
					widget.set_active(i)
				else:
					widget.set_active(0)

			elif vtype == psushi.TYPE_LIST:
				
				widget,error = self.composit_list_widget(opt, label, cValue, dataMap)
				
				if error != None:
					logging.error(error)
					continue
					
				wLabel = gtk.Label("") # no need for extra label

			elif vtype == psushi.TYPE_MAP:
				widget = self.composit_map_widget(opt, label, value, dataMap)

			else:
				logging.error(
					"PluginConfigDialog: Wrong type given: %d" % (vtype))


			self.table.attach(wLabel, 0, 1, rowCount, rowCount+1)
			self.table.attach(widget, 1, 2, rowCount, rowCount+1)

			rowCount += 1

		self._data_map = dataMap


	def save(self):
		""" save the changes made in the plugin's config section """

		cSection = plugins.get_plugin_config_section(
				self.plugin_name)
		config.create_section(cSection)

		for (key, value) in self._data_map.items():

			logging.debug("PluginConfigDialog: Result: %s -> %s" % (
					key, str(value)))
			config.set(cSection, key, str(value))




# coding: UTF-8
"""
Copyright (c) 2008 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import gtk
from gettext import gettext as _

from .. import config
from .. import gui

from ..com import sushi, NoSushiError
from ..helper.singleton import SingletonMeta


def get_ident(serverTab, nick):
	""" retrieve the ident string of a given nick """
	from_str = sushi.user_from(serverTab.name, nick)
	try:
		(names, host) = from_str.split("@")
		(nick, ident) = names.split("!")
	except ValueError:
		return None
	return ident


class NickListMenu(object):

	__metaclass__ = SingletonMeta

	def __init__(self):
		self.menu = None
		self.widgets = gui.builder.load_menu("nickListMenu")
		self.deactivate_handler = []

		if not self.widgets:
			gui.mgmt.show_inline_message(
				_("Widget creation failed."),
				_("tekka failed to create the nicklist menu.\n"
				  "It's possible that there are files missing. "
				  "Check if you have appropriate permissions to "
				  "access all files needed by tekka and restart tekka."),
				dtype="error")
			return

		sigdic = {
			"nickListMenu_deactivate_cb" : self.deactivate_cb,
			"ignoreItem_toggled_cb" : self.ignoreItem_toggled_cb,
			"kickItem_activate_cb" : self.kickItem_activate_cb,
			"banItem_activate_cb" : self.banItem_activate_cb,
			"whoisItem_activate_cb" : self.whoisItem_activate_cb,
			"sendFileItem_activate_cb" : self.sendFileItem_activate_cb,

			# modes
			"deVoiceItem_activate_cb" : self.deVoiceItem_activate_cb,
			"voiceItem_activate_cb" : self.voiceItem_activate_cb,
			"deHalfOpItem_activate_cb" : self.deHalfOpItem_activate_cb,
			"halfOpItem_activate_cb" : self.halfOpItem_activate_cb,
			"deOpItem_activate_cb" : self.deOpItem_activate_cb,
			"opItem_activate_cb" : self.opItem_activate_cb
		}

		self.widgets.connect_signals(sigdic)

		self.menu = self.widgets.get_object("nickListMenu")

	def get_menu(self, currentNick):
		""" return the menu customized menu, fit to the needs of pointedTab """
		if not self.menu or not currentNick:
			return None

		self.current_nick = currentNick

		headerItem = gtk.MenuItem(label=currentNick, use_underline=False)
		self.menu.insert(headerItem, 0)
		headerItem.show()

		serverTab = gui.tabs.get_current_tabs()[0]
		ident = get_ident(serverTab, currentNick)

		if ident and ident in sushi.ignores(serverTab.name):
			self.widgets.get_object("ignoreItem").set_active(True)

		if sushi.remote:
			self.widgets.get_object("sendFileItem").set_sensitive(False)

		self.deactivate_handler.append(
			(lambda menu,header: menu.remove(header), headerItem))

		return self.menu

	def deactivate_cb(self, menu):
		for handler in self.deactivate_handler:
			if handler[1:]:
				handler[0](menu, *handler[1:])
			else:
				handler[0](menu)

		self.deactivate_handler = []

	def ignoreItem_toggled_cb(self, item):
		serverTab = gui.tabs.get_current_tabs()[0]
		ident = get_ident(serverTab, self.current_nick)

		# FIXME: this is a "bug" in maki, ident can be None
		if not ident: return

		if item.get_active():
			if ident in sushi.ignores(serverTab.name):
				# nothing new, abort
				return
			sushi.ignore(serverTab.name, ident)
			gui.mgmt.show_inline_message(
				_("Ignoring User %(user)s") % {"user":self.current_nick},
				"",
				"info")
		else:
			sushi.unignore(serverTab.name, ident)
			gui.mgmt.show_inline_message(
				_("User %(user)s is unignored") % {"user":self.current_nick},
				"",
				"info")

	def kickItem_activate_cb(self, item):
		sTab,cTab = gui.tabs.get_current_tabs()

		if not cTab or not cTab.is_channel():
			return

		sushi.kick(sTab.name, cTab.name, self.current_nick, "")

	def banItem_activate_cb(self, item):
		sTab,cTab = gui.tabs.get_current_tabs()

		if not cTab or not cTab.is_channel():
			return

		sushi.mode(sTab.name, cTab.name, "+b %s*!*@*" % (self.current_nick) )

	def whoisItem_activate_cb(self, item):
		sTab,cTab = gui.tabs.get_current_tabs()

		if not sTab:
			return

		if config.get_bool("tekka", "whois_dialog"):
			try:
				gui.dialogs.show_dialog("whois", sTab.name,
					self.current_nick, need_sushi = True)
			except NoSushiError as e:
				gui.mgmt.show_inline_message(
					_("No connection to maki."),
					e.args[0],
					dtype="error")

		else:
			sushi.sushi.whois(sTab.name, self.current_nick)

	def sendFileItem_activate_cb(self, item):
		sTab,cTab = gui.tabs.get_current_tabs()

		def dialog_response_cb(dialog, id):
			if id == gtk.RESPONSE_OK:
				file = dialog.get_filename()
				if not file:
					gui.mgmt.show_error_dialog(
						title = _("No file selected"),
						message = _("You didn't select a file to send. Aborting."))
				else:
					sushi.dcc_send(sTab.name, self.current_nick, file)
			dialog.destroy()

		d = gtk.FileChooserDialog(
			title = _("Choose a file to send to %(nick)s" % {"nick": self.current_nick}),
			buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		d.connect("response", dialog_response_cb)
		d.show()

	def deVoiceItem_activate_cb(self, item):
		sTab,cTab = gui.tabs.get_current_tabs()

		if not cTab or not cTab.is_channel():
			return

		sushi.mode(sTab.name, cTab.name, "-v %s" % (self.current_nick) )

	def voiceItem_activate_cb(self, item):
		sTab,cTab = gui.tabs.get_current_tabs()

		if not cTab or not cTab.is_channel():
			return

		sushi.mode(sTab.name, cTab.name, "+v %s" % (self.current_nick) )

	def deHalfOpItem_activate_cb(self, item):
		sTab,cTab = gui.tabs.get_current_tabs()

		if not cTab or not cTab.is_channel():
			return

		sushi.mode(sTab.name, cTab.name, "-h %s" % (self.current_nick) )

	def halfOpItem_activate_cb(self, item):
		sTab,cTab = gui.tabs.get_current_tabs()

		if not cTab or not cTab.is_channel():
			return

		sushi.mode(sTab.name, cTab.name, "+h %s" % (self.current_nick) )

	def deOpItem_activate_cb(self, item):
		sTab,cTab = gui.tabs.get_current_tabs()

		if not cTab or not cTab.is_channel():
			return

		sushi.mode(sTab.name, cTab.name, "-o %s" % (self.current_nick) )

	def opItem_activate_cb(self, item):
		sTab,cTab = gui.tabs.get_current_tabs()

		if not cTab or not cTab.is_channel():
			return

		sushi.mode(sTab.name, cTab.name, "+o %s" % (self.current_nick) )




# coding: UTF-8
"""
Copyright (c) 2008 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import gtk
import gobject
import logging

from ..typecheck import types

COLUMN_PREFIX=0
COLUMN_NICK=1
COLUMN_AWAY=2

class NickListStore(gtk.ListStore):
	"""
	Store class for the nickList widget.
	Stores the prefix and the nick name
	in a list:
	<prefix_0>,<nick_0>
	<prefix_1>,<nick_1>
	...

	prefix is a item of NickListStore.modes,
	nick is a string.
	"""

	def __init__(self, nicks=None, prefixes=None, aways=None):
		gtk.ListStore.__init__(self, gobject.TYPE_STRING,
			gobject.TYPE_STRING, gobject.TYPE_BOOLEAN)

		self.__modes = []
		# user count, operator count, dowehaveoperators?
		self.__count = 0
		self.__opcount = 0

		if nicks and prefixes and len(nicks) == len(prefixes):
			self.add_nicks(nicks, prefixes)

		if aways and len(aways) == len(nicks):
			self.set_aways(nicks, aways)

	def __len__(self):
		return self.__count

	def get_operator_count(self):
		return self.__opcount

	def set_modes(self, modes):
		self.__modes = list(modes)
		self.__modes.append("")

	@types(needle = basestring)
	def find_nick_row(self, needle):
		for row in self:
			if row[COLUMN_NICK].lower() == needle.lower():
				return row
		return None

	def set_aways(self, nicks, aways):
		if not nicks or not aways:
			return

		for i in range(len(nicks)):
			self.set_away(nicks[i], aways[i])

	def add_nicks(self, nicks, prefixes):
		"""
		Adds a list of nicks to the NickListStore.
		After adding all nicks sortNicks is called.
		"""
		if not nicks or not prefixes:
			return

		for i in range(len(nicks)):
			self.append_nick(nicks[i], sort=False)
			self.set_prefix(nicks[i], prefixes[i], sort=False)

		self.sort_nicks()

	def has_nick(self, nick):
		return nick in self.get_nicks()

	def get_nicks(self):
		"""
		returns all nick names stored
		"""
		return [l[COLUMN_NICK] for l in self if l is not None ]

	def get_nicks_mode(self):
		""" return a tuple per nick with (prefix,nick) """
		return [
			(l[COLUMN_PREFIX],l[COLUMN_NICK]) for l in self
			if l]

	def append_nick(self, nick, sort=True):
		"""
		appends a nick to the store, if sort is false,
		data in the NickListStore would'nt be sorted in-place
		"""
		iter = self.append(None)

		self.set(iter, COLUMN_NICK, nick)
		self.set(iter, COLUMN_PREFIX, "")
		self.set(iter, COLUMN_AWAY, False)

		self.__count += 1

		if sort:
			self.sort_nicks()

	def modify_nick(self, nick, newnick):
		"""
		renames the nick `nick` to `newnick`
		"""
		row = self.find_nick_row(nick)

		if row:
			self.set(row.iter, COLUMN_NICK, newnick)
			self.sort_nicks()

	def remove_nick(self, nick):
		"""
		removes the whole column where nick name = `nick`
		"""
		store = self
		row = self.find_nick_row(nick)

		if not row:
			return

		if row[COLUMN_PREFIX] in self.__modes[:-2]:
			self.__opcount -= 1

		self.__count -= 1

		store.remove(row.iter)

	def clear(self, count_reset = True):
		"""
		remove all entries from the store
		and set counts to 0.
		"""
		gtk.ListStore.clear(self)

		if count_reset:
			self.__count = 0
			self.__opcount = 0

	def set_away(self, nick, away):
		row = self.find_nick_row(nick)

		if not row:
			return

		row[COLUMN_AWAY] = away

	def set_prefix(self, nick, prefix, sort=True):
		"""
		sets the prefix `prefix` to the nick `nick`.
		After setting the prefix and sort is true
		the data in the NickListStore will be sorted
		in place.
		"""
		row = self.find_nick_row(nick)

		if not row:
			return

		# list without voice and no-mode
		op_pre = self.__modes[:-2]

		if row[COLUMN_PREFIX] in op_pre and prefix not in op_pre:
			# op goes to non-op
			self.__opcount -= 1

		elif row[COLUMN_PREFIX] not in op_pre and prefix in op_pre:
			# wasn't an op and becomes one
			self.__opcount += 1

		row[COLUMN_PREFIX] = prefix

		if sort:
			self.sort_nicks()

	def get_prefix(self, nick):
		""" returns the prefix for the nick identified
			by `nick`. If the nick is not found
			None is returned.
		"""
		row = self.find_nick_row(nick)

		if not row:
			# this is possible if outer-channel-talking is allowed
			return ""

		return row[COLUMN_PREFIX]

	def search_nick(self, needle):
		""" returns a list of nicks wich are beginning with
			the string `needle`
		"""
		return [l[COLUMN_NICK] for l in self
			if l and l[COLUMN_NICK][0:len(needle)].lower()==needle]

	def search_nick_by_prefix(self, prefixes):
		""" Searches for nicks which prefix is in the tuple prefixes
			and returns the found nicks as a list.
		"""
		return [l[COLUMN_NICK] for l in self
			if l and l[COLUMN_PREFIX] in prefixes]

	def sort_nicks(self):
		""" sort the NickListStore in-place by prefix and
			then by nick name.
			Use insertion sort.
		"""
		def retreive_prefix(row):
			try:
				return self.__modes.index(row[COLUMN_PREFIX])
			except ValueError:
				logging.error(
					"sort_nicks: prefix '%s' (%s) not in modes (%s)" % (
					row[COLUMN_PREFIX], row[COLUMN_NICK], self.__modes))
				return None

		def compare(left, current):
			# left >? current

			pLeft = retreive_prefix(left)
			pRight = retreive_prefix(current)

			# pLeft < pRight => Left > Right
			# smallest prefix index p is highest mode

			d = lambda: cmp(left[COLUMN_NICK].lower(),
				current[COLUMN_NICK].lower())==1

			return (pLeft > pRight) or (pLeft == pRight and d())

		for j in range(len(self)):

			i = j - 1
			current = [self[j][0],self[j][1],self[j][2]]

			while i > -1 and compare(self[i], current):
				self.set(self[i+1].iter, 0, self[i][0], 1, self[i][1], 2, self[i][2])
				i -= 1

			self.set(self[i+1].iter, 0, current[0], 1, current[1], 2, current[2])



""" provide functions for history handling.
	note that history stuff is only possible
	if maki is not running remote.
"""
import re
import os
from ..com import sushi
from ..typecheck import types

FILEPATTERN= re.compile(r'([0-9]+)-([0-9]+)\.txt')
DATEPATTERN= re.compile(r'^([0-9]+)-([0-9]+)-([0-9]+) ([0-9]+):([0-9]+):([0-9]+)')

def get_log_dir():
	return sushi.config_get("directories","logs")


def get_available_servers(force_remote=False):
	""" return a list with all servers containing logs """
	if not force_remote and sushi.remote:
		return []

	log_dir = get_log_dir()

	if not os.path.exists(log_dir):
		return []

	return [dir for dir in os.listdir(log_dir) if os.path.isdir(
									os.path.join(log_dir, dir))]


@types(server=basestring)
def get_available_conversations(server, force_remote=False):
	""" return a list with all available logged channels/queries """
	if (not force_remote and sushi.remote) or not server:
		return []

	log_dir = os.path.join(get_log_dir(), server)

	if not os.path.exists(log_dir) or not os.path.isdir(log_dir):
		return []

	return [dir for dir in os.listdir(log_dir) if os.path.isdir(
											   os.path.join(log_dir, dir))]


@types(server=basestring, target=basestring)
def get_available_logs(server, target, force_remote=False):
	""" return a list with all available logs for the target """
	if (not force_remote and sushi.remote) or not server or not basestring:
		return []

	log_dir = os.path.join(get_log_dir(), server, target)

	if not os.path.exists(log_dir):
		return []

	return [f for f in os.listdir(log_dir) if FILEPATTERN.match(f)
											and os.path.isfile(
												os.path.join(log_dir, f))]

@types(log_file=basestring)
def get_log_date(log_file):
	""" return (year,month) tuple """
	match = FILEPATTERN.match(log_file)

	if not match:
		return ""

	return [int(n) for n in match.groups()]


@types(server=basestring, target=basestring, log=basestring)
def get_log_path(server, target, log):
	if not server or not target or not log:
		return ""
	return os.path.join(get_log_dir(), server, target, log)


@types(fd=file)
def parse_day_offsets(fd):
	offsets = {}

	start = fd.tell()
	offset = fd.tell()
	last_day = 0

	year = month = day = hour = minute = second = 0

	for line in fd:
		match = DATEPATTERN.match(line)

		if not match:
			continue

		(year, month, day, hour, minute, second) = [int(n)
				for n in match.groups()]

		if day != last_day:
			offsets[(year, month, last_day)] = (start, offset)
			last_day = day
			start = offset

		offset += len(line)

	if last_day != 0:
		offsets[(year, month, last_day)] = (start, offset)

	return offsets



# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""
from gettext import gettext as _

import string # is_highlight
import inspect
import time
import urwid
import logging

import config
import tabs
import connection

import helper.markup
import helper.code

from connection import parse_from, sushi

from messages import print_tab, print_error, \
	format_message, print_tab_notification

signals = {}
_backup_signals = []

def connect_signal (signal, handler):
	""" connect handler to signal """
	global signals
	if type(signal) != type(""):
		raise TypeError

	if not signals.has_key (signal):
	  	signals[signal] = {}

	if signals[signal].has_key(handler):
		# no doubles
		return

	signals[signal][handler] = sushi.connect_to_signal (signal, handler)

def disconnect_signal (signal, handler):
	""" disconnect handler from signal """
	global signals
	if type(signal) != type(""):
		raise TypeError

	try:
		ob = signals[signal][handler]
	except KeyError:
		return
	else:
		ob.remove()
		del signals[signal][handler]


def setup(mw):
	""" called by main, mw is the main_window """
	global main_window
	main_window = mw

	urwid.connect_signal(connection.sushi, "connected", maki_connected)
	urwid.connect_signal(connection.sushi, "disconnected", maki_disconnected)

def maki_disconnected(sushi):
	global signals, _backup_signals

	for signal in signals:
		for handler in signals[signal]:
			signals[signal][handler].remove()

			_backup_signals.append((signal,handler))
	signals = {}

def _reconnect_signals():
	global _backup_signals

	for (signal, handler) in _backup_signals:
		connect_signal(signal, handler)
	_backup_signals = []

def maki_connected(sushi_interface):
	""" called by dbus connection handler.
		sushi is the dbus interface
	"""
	global _backup_signals

	if _backup_signals:
		# restore saved signal/handler pairs
		_reconnect_signals()

	else:
		# initially setup signals

		# message receiving
		connect_signal("action", sushi_action)
		connect_signal("message", sushi_message)
		connect_signal("ctcp", sushi_ctcp)
		connect_signal("notice", sushi_notice)

		# action signals
		connect_signal("invite", sushi_invite)
		connect_signal("join", sushi_join)
		connect_signal("kick", sushi_kick)
		connect_signal("nick", sushi_nick)
		connect_signal("mode", sushi_mode)
		connect_signal("oper", sushi_oper)
		connect_signal("part", sushi_part)
		connect_signal("quit", sushi_quit)
		connect_signal("topic", sushi_topic)

		# informative signals
		connect_signal("banlist", sushi_banlist)
		connect_signal("list", sushi_list)
		connect_signal("motd", sushi_motd)
		connect_signal("names", sushi_names)
		connect_signal("whois", sushi_whois)
		connect_signal("error", sushi_error)

		# status signals
		connect_signal("connect", sushi_connect_attempt)
		connect_signal("connected", sushi_connected)
		connect_signal("away_message", sushi_away_message)
		connect_signal("user_away", sushi_user_away)
		connect_signal("shutdown", sushi_shutdown)
		connect_signal("dcc_send", sushi_dcc_send)

	setup_connected_servers()

def setup_connected_servers():
	for server in sushi.servers():
		stab = main_window.find_server(server)

		if stab:
			stab.set_connected(True)
			continue

		stab = tabs.Server(name = str(server))
		main_window.add_server(stab)

		sushi.nick(server, "")

		for channel in sushi.channels(server):
			ctab = main_window.find_tab(server, channel)

			if ctab:
				ctab.set_joined(True)
				sushi.topic(server, channel, "")
				continue

			ctab = tabs.Channel(name = channel,
				parent = stab)
			ctab.set_joined(True)

			ctab.print_last_log()

			sushi.names(server, channel)

			topic = sushi.channel_topic(server, channel)

			sushi_topic(time.time(), server, "", channel, topic)

		stab.set_connected(True)

	main_window.update_divider()

####################################################

def find_parent_tab(server, target):
	parent = main_window.find_server(server)
	if not parent:
		print_error("parent server '%s' for '%s' not found." % (
			server, target))
		return None
	return parent

def no_tab_error(msg):
	def dec(fun):
		def subdec(*args, **kwargs):
			parameters = inspect.getargspec(fun)[0]
			param_dict = {}

			for i in range (len(args[:len(parameters)])):
				param_dict[parameters[i]] = args[i]
			param_dict.update (kwargs)

			tab = fun(*args, **kwargs)
			if not tab:
				print_error(msg % param_dict)

			return tab
		return subdec
	return dec

@no_tab_error ("No tab found for %(target)s on %(server)s. (nick = %(nick)s)")
def find_target_tab(server, target, nick):
	""" server: current server,
		target: target of the action,
		nick: causes the action

		Return a query-tab if target is a user,
		return a channel-tab if target is a channel
	"""
	if not target[0] in sushi.support_chantypes(server):
		# we got a query here
		return find_query_tab(server, nick, create = True)

	else:
		return find_channel_tab(server, target, create = False)
	return None

@no_tab_error ("No tab for %(partner)s on %(server)s found.")
def find_query_tab(server, partner, create = True):
	tab = main_window.find_tab(server, partner)

	if tab:
		return tab

	if not create:
		return None

	parent = find_parent_tab(server, partner)

	if not parent:
		return None

	tab = tabs.Query(name = partner, parent = parent)
	return tab

@no_tab_error ("No tab for channel %(channel)s on %(server)s found.")
def find_channel_tab(server, channel, create = False):
	tab = main_window.find_tab(server, channel)

	if tab:
		return tab

	if not create:
		return None

	parent = find_parent_tab(server, channel)

	if not parent:
		return None

	tab = tabs.Channel(name = channel, parent = parent)
	return tab

def is_highlighted (server, text):
	def has_highlight(text, needle):
		punctuation = string.punctuation + " \n\t"
		ln = len(needle)
		for line in text.split("\n"):
			line = line.lower()
			i = line.find(needle)
			if i >= 0:
				if (line[i-1:i] in punctuation
				and line[ln+i:ln+i+1] in punctuation):
					return True
		return False

	server_tab = main_window.find_server(server)
	highlightwords = config.get_list("chatting", "highlight_words", [])
	highlightwords.append(server_tab.get_nick())

	for word in highlightwords:
		if has_highlight(text, word):
			return True
	return False

def current_server_tab_print(server, message):
	""" server: basestring
		message: basestring | FormattedMessage

		for server related notifications.
		print the message to the current active tab
		if the tab belongs to the given server.
		If no tab of the given server is active, print
		it to the server tab.
	"""
	current_tab = main_window.current_tab

	if (not current_tab
		or (type(current_tab) == tabs.Server and current_tab.name != server)
		or (current_tab.parent and current_tab.parent.name != server)):
		# print it to the server tab
		tab = main_window.find_server(server)
		print_tab(tab, message)
	else:
		print_tab(current_tab, message)


def color_nick_markup_cb(msg,values=["nick"]):
	def get_nick_color(nick):
		colors = main_window._palette[14:14+8]
		i = sum([ord(n) for n in nick]) % len(colors)
		return colors[i][0]

	if msg.own:
		return msg._markup()
	else:
		for val in msg.values:
			# Escape all values
			msg.values[val] = msg.values[val].replace(
				"\\","\\\\").replace("'","\\'")

			# Build markup tuples around values
			# This substitutes every input value to the form
			#   '),('color','value'),('base_color',
			# and embraces the generated template like this
			#   [('base_color','unicode(msg)')]
			# the following form is achieved
			#   [('base_color','Text '),('color','value'),
			#   	('base_color','...')]
			# repr is used for escaping.
			if val in values:
				msg.values[val] = "'),(%s,%s),(%s,u'" % (
					repr(get_nick_color(msg.values[val])),
					repr(msg.values[val]),
					repr(msg.base_color))
		x = "[(%s,u'%s')]" % (repr(msg.base_color),unicode(msg))
		return eval(x)

# message signals

def sushi_action(time, server, sender, target, message):
	nick = parse_from(sender)[0]
	tab = find_target_tab(server, target, nick)

	if not tab:
		return

	msg = format_message("messages", "action",
		{"nick": nick,
		 "message": message},
		own = (tab.parent.get_nick() == nick),
		highlight = (nick != tab.parent.get_nick()) and is_highlighted(server, message))

	print_tab(tab, msg)


def sushi_message(time, server, sender, target, message):
	nick = parse_from(sender)[0]
	tab = find_target_tab(server, target, nick)

	if not tab:
		return

	msg = format_message("messages", "message",
		{"message": message,
		 "nick": nick,
		 "prefix": sushi.user_channel_prefix(server, target, nick)},
		own = (nick == tab.parent.get_nick()),
		highlight = (nick != tab.parent.get_nick())
					and is_highlighted(server, message))

	msg.markup_cb = color_nick_markup_cb

	print_tab(tab, msg)

def sushi_ctcp(time, server, sender, target, message):
	own_nick = main_window.find_server(server).get_nick()
	nick = parse_from(sender)[0]

	msg = format_message("messages", "ctcp",
		{"nick": nick,
		 "target": target,
		 "message": message},
		own = (nick == own_nick),
		highlight = is_highlighted(server, message))

	msg.markup_cb = color_nick_markup_cb

	if target[0] in sushi.support_chantypes(server):
		tab = find_target_tab(server, target)
		if not tab:
			current_server_tab_print(server, msg)
	else:
		current_server_tab_print(server, msg)

def sushi_notice(time, server, sender, target, message):
	own_nick = main_window.find_server(server).get_nick()
	nick = parse_from(sender)[0]

	msg = format_message("messages", "notice",
		{"nick": nick,
		 "target": target,
		 "message": message},
		own = (nick == own_nick),
		highlight = is_highlighted(server, message))

	msg.markup_cb = color_nick_markup_cb

	if target[0] in sushi.support_chantypes(server):
		tab = find_target_tab(server, target)
		if not tab:
			current_server_tab_print(server, msg)
	else:
		current_server_tab_print(server, msg)

# action signals

def sushi_invite(time, server, sender, channel, who):
	""" sender can be empty """
	sender_name = parse_from(sender)[0]

	msg = format_message("actions", "invite",
		{"nick": sender_name,
		 "who": who,
		 "channel": channel},
		own = (sender_name == main_window.find_server(server).get_nick()),
		highlight = (who == sender_name))

	msg.markup_cb = color_nick_markup_cb

	current_server_tab_print(server, msg)

def sushi_join(time, server, sender, channel):

	server_tab = main_window.find_server(server)

	if not server_tab:
		print_error("Missing server tab for '%s'." % (server))
		return

	tab = main_window.find_tab(server, channel)
	nick = parse_from(sender)[0]

	if nick == server_tab.get_nick():
		# we join
		if not tab:
			tab = tabs.Channel(name = channel, parent = server_tab)

		tab.set_joined(True)
		main_window.update_divider()

	else:
		# somebody joined
		tab.nicklist.add_nick(nick)

	msg = format_message("actions", "join",
		{"nick": nick,
   		 "host": sender,
		 "channel": channel},
		own = (nick == server_tab.get_nick()),
		highlight = False)

	msg.markup_cb = color_nick_markup_cb

	print_tab(tab, msg)


def sushi_kick(time, server, sender, channel, who, message):
	""" message can be empty """
	stab = main_window.find_server(server)

	if not stab:
		print_error("missing stab '%s'" % (server))
		return

	nick = parse_from(sender)[0]
	tab = main_window.find_tab(server, channel)

	if who == stab.get_nick():
		# we got kicked
		pass

	else:
		tab.nicklist.remove_nick(who)

	msg = format_message("actions", "kick",
		{"who": who,
		 "channel": channel,
		 "nick": nick,
		 "reason": message},
		own = (who == stab.get_nick()),
		highlight = (nick == tab.parent.get_nick()))
	msg.markup_cb = color_nick_markup_cb

	print_tab(tab, msg)

def sushi_nick(time, server, old, new):
	""" old == "" => new == current nick """
	stab = main_window.find_server(server)

	if not stab:
		print_error("missing stab '%s'" % server)
		return

	if not old or old == stab.get_nick():
		stab.set_nick(new)

		if main_window.current_tab in tabs.tree_to_list([stab]):
			main_window.update_divider()

		msg = format_message("actions", "nick",
			{"nick": old,
			 "new_nick": new},
			own = True,
			highlight = False)

		current_server_tab_print(server, msg)

	else:
		# print status message in channel tab
		old = connection.parse_from(old)[0]
		new = connection.parse_from(new)[0]

		msg = format_message("actions", "nick",
			{"nick": old,
			 "new_nick": new},
			own = False,
			highlight = False)

		msg.markup_cb = color_nick_markup_cb
		msg.markup_cb_kwargs = {"values":["nick","new_nick"]}

		for tab in stab.children:
			if type(tab) == tabs.Channel and tab.nicklist.has_nick(old):
				tab.nicklist.rename_nick(old,new)
				print_tab(tab, msg)

def sushi_mode(time, server, sender, target, mode, param):
	""" sender can be empty, param can be empty """
	if sender == "":
		# mode listing
		msg = format_message("actions", "mode_list",
			{"target": target, "mode": mode},
			own = (target == main_window.find_server(server).get_nick()),
			highlight = False)
		current_server_tab_print(server, msg)

	else:
		nick = parse_from(sender)[0]
		msg = format_message("actions", "mode",
			{"nick": nick,
			 "mode": mode,
			 "param": param and " "+param or "",
			 "target": target},
			own = (nick == main_window.find_server(server).get_nick()),
			highlight = (target == main_window.find_server(server).get_nick()))
		msg.markup_cb = color_nick_markup_cb

		tab = main_window.find_tab(server, target)
		print_tab(tab, msg)

def sushi_oper(time, server):
	msg = format_message("actions", "oper",
		{"nick": main_window.find_server(server).get_nick()},
		own = True,
		highlight = False)
	current_server_tab_print(server, msg)

def sushi_part(time, server, sender, channel, message):
	""" message can be empty """
	tab = main_window.find_tab (server, channel)
	nick = parse_from(sender)[0]

	if not tab:
		# ignore this cause it could be this tab was
		# closed recently
		return

	if tab.parent.get_nick() == nick:
		# we parted
		tab.set_joined(False)
		main_window.update_divider()

	else:
		tab.nicklist.remove_nick(nick)

	msg = format_message("actions", "part",
		{"nick": nick,
		 "channel": channel,
		 "reason": message},
		own = (tab.parent.get_nick() == nick),
		highlight = False)
	msg.markup_cb = color_nick_markup_cb

	print_tab(tab, msg)

def sushi_quit(time, server, sender, message):
	""" message can be empty """
	server_tab = main_window.find_server(server)

	if not server_tab:
		print_error("No server tab for server '%s'." % (server))
		return

	nick = parse_from(sender)[0]

	msg = format_message("actions", "quit",
		{"nick": nick,
		 "reason": message},
		own = (nick == server_tab.get_nick()),
		highlight = False)
	msg.markup_cb = color_nick_markup_cb

	if nick == server_tab.get_nick():
		# we quit
		server_tab.set_connected(False)

		for child in server_tab.children:
			print_tab(child, msg)

	else:
		for tab in server_tab.children:
			if type(tab) == tabs.Channel and tab.nicklist.has_nick(nick):
				tab.nicklist.remove_nick(nick)
				print_tab(tab, msg)
			elif (type(tab) == tabs.Query
				and tab.name.lower() == nick.lower()):
				print_tab(tab, msg)

def sushi_topic(time, server, sender, channel, topic):
	""" sender can be empty """
	tab = main_window.find_tab(server, channel)

	if not tab:
		print_error("No tab '%s' on '%s' for topic setting." % (
			channel, server))
		return

	tab.set_topic(topic)
	nick = parse_from(sender)[0]

	if sender == "":
		template = "topic_anonymous"
	else:
		template = "topic"

	msg = format_message("actions", template,
		{"nick": nick,
		 "channel": channel,
		 "topic": topic},
		own = (tab.parent.get_nick() == nick),
		highlight = False)
	if nick:
		msg.markup_cb = color_nick_markup_cb

	print_tab(tab, msg)

	if tab == main_window.current_tab:
		# update topic bar
		main_window.header.set_text(tab.get_topic())

# informative signals

def sushi_banlist(time, server, channel, mask, who, when):
	""" who can be empty (""), when can be empty (0).
		mask == "" and who == "" and when == -1 => EOL
	"""
	self = helper.code.init_function_attrs(sushi_banlist, first_run=True)

	if "" in (mask,who) and when == -1:
		# EOBL
		self.first_run = True

		msg = format_message("informative", "banlist_end",
			{"channel": channel},
			own = False,
			highlight = False)

		print_tab(self.tab, msg)
	elif self.started:
		# Listing just started

		self.first_run = False

		self.tab = main_window.find_channel_tab(server,channel)

		if not self.tab:
			# SHOULD NOT HAPPEN (banlist only in active channels)
			return

		msg = format_message("informative", "banlist_begin",
			{"channel": channel},
			own = False,
			highlight = False)

		print_tab(self.tab, msg)
	else:
		# Banlist item

		msg = format_message("informative", "banlist_begin",
			{"who": who,
			 "mask": mask,
			 "when": when},
			own = False,
			highlight = False)

		print_tab(self.tab, msg)

def sushi_list(time, server, channel, users, topic):
	""" channel == "" and users == -1 and topic == "" => EOL """
	pass

def sushi_motd(time, server, message):
	""" message == "" => End OR no MOTD """
	if message == "":
		# fetch support data
		server_tab = main_window.find_server(server)
		server_tab.update()
	pass

def sushi_names(time, server, channel, nicks, prefixes, _call_count = {"n":0}):
	""" len(nicks) == 0 => EOL """
	# TODO: colorize

	tab = find_channel_tab(server, channel)

	if not tab:
		return

	if _call_count["n"] == 0:
		msg = format_message("informative", "names_begin",
			{"channel": channel})
		print_tab(tab, msg)

	if len(nicks) != 0:
		max_width = 7
		width = 0
		msg = ""
		for nick in nicks:
			tab.nicklist.add_nick(nick)

			prefix = prefixes[width]
			msg += prefix+nick+" "

			if (width+1) % max_width == 0 or (width+1) == len(nicks):
				msg = msg[:-1]
				msg = format_message("informative", "names_item",
					{"row": msg})
				print_tab(tab, msg)
				msg = ""

			width += 1

		_call_count["n"] += 1

	else:
		msg = format_message("informative", "names_end", {})
		print_tab(tab, msg)
		_call_count["n"] = 0

def sushi_whois(time, server, nick, message):
	""" message == "" => EOL """
	pass

def sushi_error(time, server, domain, reason, arguments):

	def noSuch(time, server, target, type):
		if type == "nick":
			# no such nick/channel
			pass
		elif type == "server":
			# no such server
			pass
		elif type == "channel":
			# no such channel
			pass

	def cannotJoin(time, server, channel, reason):
		""" The channel could not be joined.
			reason : { l (full), i (invite only), b (banned), k (key) }
		"""
		if reason == "full":
			# channel is full
			pass
		elif reason == "invite":
			# channel is invite only
			pass
		elif reason == "banned":
			# you're bannend
			pass
		elif reason == "key":
			# needs key
			pass

	if domain == "no_such":
		noSuch(time, server, arguments[0], reason)

	elif domain == "cannot_join":
		cannotJoin(time, server, arguments[0], reason)

	elif domain == "privilege":
		if reason == "channel_operator":
			# need op
			pass

# status signals

def sushi_connect_attempt(time, server):
	tab = main_window.find_server(server)

	if not tab:
		tab = tabs.Server(name = server)

		main_window.add_server(tab)

		print_tab_notification(tab, "Connecting...")

	elif tab.connected:
		tab.set_connected(False)

		print_tab_notification(tab, "Reconnecting...")

	else:
		print_tab_notification(tab, "Connecting...")

def sushi_connected(time, server):
	tab = main_window.find_server(server)

	if not tab:
		tab = tabs.Server(name = server)
		main_window.add_server(tab)

	print_tab_notification(tab, "Connected.")
	tab.set_connected(True)

def sushi_away_message(time, server, nick, message):
	pass

def sushi_user_away(time, server, from_str, away):
	pass

def sushi_shutdown(time):
	pass

def sushi_dcc_send(time, id, server, sender, filename,
				size, progress, speed, status):
	""" handle dcc incoming/outgoing """
	# setup function attributes
	self = helper.code.init_function_attrs(sushi_dcc_send, new_register={})

	# import dcc states
	from helper.dcc import s_incoming, s_running, s_new

	logging.debug("dcc_send: (%d,%s,%s,%s)" % (id, server, sender, filename))

	if ("" in (server, sender, filename)
	and 0 in (size, progress, speed, status)):
		# file transfer removed
		logging.debug("filetransfer %d removed." % (id))

	else:
		if status & s_incoming:
			if status & s_new:
				logging.debug("incoming filetransfer (%d): %s %s %s" % (id, server, sender, filename))

				msg = format_message("informative", "dcc_new_incoming",
						{"sender": sender,
						 "id": id,
						 "filename": filename,
   						 "size": size})

				if main_window.current_tab:
					print_tab(main_window.current_tab, msg)
				else:
					main_window.print_text(unicode(msg))

				self.new_register[id] = True

			elif status & s_running and status & s_incoming:
				if not self.new_register.has_key(id):
					# notify about auto accepted file transfer
					msg = format_message(
							"informative",
							"dcc_file_auto_accept",
							{"sender": sender,
							 "filename": filename,
							 "size": size})

					if main_window.current_tab:
						print_tab(main_window.current_tab, msg)
					else:
						main_window.print_text(unicode(msg))

					self.new_register[id] = True






# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

from gettext import gettext as _

from tekka import com
from tekka import config
from tekka import commands
from tekka import signals
from tekka import config
from tekka import gui

"""
TYPE_STRING: Takes one argument (default string)
TYPE_PASSWORD: Hidden string. Takes one argument (default string)
TYPE_NUMBER: Takes one argument (default number)
TYPE_BOOL: Takes one argument (default bool value)
TYPE_CHOICE: Takes n key/value tuple and a default index.
TYPE_LIST: Takes n values in a tuple.
TYPE_MAP: Takes n key/value tuple in a tuple.
"""
(TYPE_STRING,
 TYPE_PASSWORD,
 TYPE_NUMBER,
 TYPE_BOOL,
 TYPE_CHOICE,
 TYPE_LIST,
 TYPE_MAP
) = range(7)


class Plugin (object):


	def __init__(self, plugin_name):
		self._plugin_name = plugin_name

		self.__signals = {}
		self.__commands = {}

		com.sushi.g_connect("maki-connected", self.maki_connected)
		com.sushi.g_connect("maki-disconnected", self.maki_disconnected)


	def unload(self):
		""" unregister all signals and commands used by
			this plugin
		"""

		for (signal, hlist) in self.__signals:
			for handler in hlist:
				signals.disconnect_signal(signal, handler)

		for (command, handler) in self.__commands:
			commands.remove_command(command, handler)


	def maki_connected(self, interface):
		""" hook, called if connection to maki is etablished """
		pass


	def maki_disconnected(self, interface):
		""" hook, called if connection to maki is cut """
		pass


	def display_error(self, message):

		gui.mgmt.show_inline_message(
			_("Plugin “%(plugin)s” caused an error.") % {
				"plugin": self._plugin_name
			}, "%(message)s" % {
				"message": message
			}, dtype="error")


	def get_bus(self):

		return com.sushi


	def get_nick(self, server):
		nick = com.get_own_nick(server)

		if not nick:
			return None

		return nick


	def add_command(self, command, func):

		def func_proxy (server, target, args):
			return func(server.name, target.name, args)

		if commands.add_command(command, func_proxy):
			self.__commands[command] = func_proxy
			return True

		return False


	def remove_command(self, command):

		if commands.remove_command(command):
			del self.__commands[commands]
			return True

		return False


	def connect_signal(self, signal, func):

		if signals.connect_signal(signal, func):

			if self.__signals.has_key(signal):
				self.__signals[signal].append(func)
			else:
				self.__signals[signal] = [func]
			return True

		return False


	def disconnect_signal(self, signal, func):

		if signals.disconnect_signal(signal, func):

			i = self.__signals[signal].index(func)
			del self.__signals[signal][i]
			return True

		return False


	def set_config(self, name, value):

		section = "plugin_%s" % (self._plugin_name)

		config.create_section(section)

		return config.set(section, name, value)


	def get_config(self, name, default = None):

		section = "plugin_%s" % (self._plugin_name)

		return config.get(section, name, default)


	def parse_from(self, host):
		return com.parse_from(host)
